# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm
from niapy.util import full_array, euclidean

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.other')
logger.setLevel('INFO')

__all__ = ['AnarchicSocietyOptimization', 'elitism', 'sequential', 'crossover']


def elitism(x, xpb, xb, xr, mp_c, mp_s, mp_p, mutation_rate, crossover_probability, task, rng):
    r"""Select the best of all three strategies.

    Args:
        x (numpy.ndarray): individual position.
        xpb (numpy.ndarray): individuals best position.
        xb (numpy.ndarray): current best position.
        xr (numpy.ndarray): random individual.
        mp_c (float): Fickleness index value.
        mp_s (float): External irregularity index value.
        mp_p (float): Internal irregularity index value.
        mutation_rate (float): scale factor.
        crossover_probability (float): crossover factor.
        task (Task): optimization task.
        rng (numpy.random.Generator): random number generator.

    Returns:
        Tuple[numpy.ndarray, float]:
            1. New position of individual
            2. New positions fitness/function value

    """
    xn = [task.repair(mp_current(x, mutation_rate, crossover_probability, mp_c, rng), rng=rng),
          task.repair(mp_society(x, xr, xb, crossover_probability, mp_s, rng), rng=rng),
          task.repair(mp_past(x, xpb, crossover_probability, mp_p, rng), rng=rng)]
    xn_f = np.apply_along_axis(task.eval, 1, xn)
    ib = np.argmin(xn_f)
    return xn[ib], xn_f[ib]


def sequential(x, xpb, xb, xr, mp_c, mp_s, mp_p, mutation_rate, crossover_probability, task, rng):
    r"""Sequentially combines all three strategies.

    Args:
        x (numpy.ndarray): individual position.
        xpb (numpy.ndarray): individuals best position.
        xb (numpy.ndarray): current best position.
        xr (numpy.ndarray): random individual.
        mp_c (float): Fickleness index value.
        mp_s (float): External irregularity index value.
        mp_p (float): Internal irregularity index value.
        mutation_rate (float): scale factor.
        crossover_probability (float): crossover factor.
        task (Task): optimization task.
        rng (numpy.random.Generator): random number generator.

    Returns:
        tuple[numpy.ndarray, float]:
            1. new position
            2. new positions function/fitness value

    """
    xn = task.repair(mp_society(
        mp_past(mp_current(x, mutation_rate, crossover_probability, mp_c, rng), xpb, crossover_probability, mp_p, rng),
        xr,
        xb, crossover_probability, mp_s, rng), rng=rng)
    return xn, task.eval(xn)


def crossover(x, xpb, xb, xr, mp_c, mp_s, mp_p, mutation_rate, crossover_probability, task, rng):
    r"""Create a crossover over all three strategies.

    Args:
        x (numpy.ndarray): individual position.
        xpb (numpy.ndarray): individuals best position.
        xb (numpy.ndarray): current best position.
        xr (numpy.ndarray): random individual.
        mp_c (float): Fickleness index value.
        mp_s (float): External irregularity index value.
        mp_p (float): Internal irregularity index value.
        mutation_rate (float): scale factor.
        crossover_probability (float): crossover factor.
        task (Task): optimization task.
        rng (numpy.random.Generator): random number generator.

    Returns:
        Tuple[numpy.ndarray, float]:
            1. new position
            2. new positions function/fitness value.

    """
    xns = [task.repair(mp_current(x, mutation_rate, crossover_probability, mp_c, rng), rng=rng),
           task.repair(mp_society(x, xr, xb, crossover_probability, mp_s, rng), rng=rng),
           task.repair(mp_past(x, xpb, crossover_probability, mp_p, rng), rng=rng)]
    index = rng.integers(len(xns))
    x = np.asarray([xns[index][i] if rng.random() < crossover_probability else x[i] for i in range(len(x))])
    return x, task.eval(x)


def mp_current(x, mutation_rate, crossover_rate, mp, rng):
    r"""Get bew position based on fickleness.

    Args:
        x (numpy.ndarray): Current individuals position.
        mutation_rate (float): Scale factor.
        crossover_rate (float): Crossover probability.
        mp (float): Fickleness index value
        rng (numpy.random.Generator): Random number generator

    Returns:
        numpy.ndarray: New position

    """
    if mp < 0.5:
        b = np.sort(rng.choice(len(x), 2, replace=False))
        x[b[0]:b[1]] = x[b[0]:b[1]] + mutation_rate * rng.normal(0, 1, b[1] - b[0])
        return x
    return np.asarray(
        [x[i] + mutation_rate * rng.normal(0, 1) if rng.random() < crossover_rate else x[i] for i in range(len(x))])


def mp_society(x, xr, xb, crossover_rate, mp, rng):
    r"""Get new position based on external irregularity.

    Args:
        x (numpy.ndarray): Current individuals position.
        xr (numpy.ndarray): Random individuals position.
        xb (numpy.ndarray): Global best individuals position.
        crossover_rate (float): Crossover probability.
        mp (float): External irregularity index.
        rng (numpy.random.Generator): Random number generator.

    Returns:
        numpy.ndarray: New position.

    """
    if mp < 0.25:
        b = np.sort(rng.choice(len(x), 2, replace=False))
        x[b[0]:b[1]] = xb[b[0]:b[1]]
        return x
    elif mp < 0.5:
        return np.asarray([xb[i] if rng.random() < crossover_rate else x[i] for i in range(len(x))])
    elif mp < 0.75:
        b = np.sort(rng.choice(len(x), 2, replace=False))
        x[b[0]:b[1]] = xr[b[0]:b[1]]
        return x
    return np.asarray([xr[i] if rng.random() < crossover_rate else x[i] for i in range(len(x))])


def mp_past(x, xpb, crossover_rate, mp, rng):
    r"""Get new position based on internal irregularity.

    Args:
        x (numpy.ndarray): Current individuals position.
        xpb (numpy.ndarray): Current individuals personal best position.
        crossover_rate (float): Crossover probability.
        mp (float): Internal irregularity index value.
        rng (numpy.random.Generator): Random number generator.

    Returns:
        numpy.ndarray: Current individuals new position.

    """
    if mp < 0.5:
        b = np.sort(rng.choice(len(x), 2, replace=False))
        x[b[0]:b[1]] = xpb[b[0]:b[1]]
        return x
    return np.asarray([xpb[i] if rng.random() < crossover_rate else x[i] for i in range(len(x))])


class AnarchicSocietyOptimization(Algorithm):
    r"""Implementation of Anarchic Society Optimization algorithm.

    Algorithm:
        Anarchic Society Optimization algorithm

    Date:
        2018

    Authors:
        Klemen Berkovič

    License:
        MIT

    Reference paper:
        Ahmadi-Javid, Amir. "Anarchic Society Optimization: A human-inspired method." Evolutionary Computation (CEC), 2011 IEEE Congress on. IEEE, 2011.

    Attributes:
        Name (list of str): List of stings representing name of algorithm.
        alpha (List[float]): Factor for fickleness index function :math:`\in [0, 1]`.
        gamma (List[float]): Factor for external irregularity index function :math:`\in [0, \infty)`.
        theta (List[float]): Factor for internal irregularity index function :math:`\in [0, \infty)`.
        d (Callable[[float, float], float]): function that takes two arguments that are function values and calculates the distance between them.
        dn (Callable[[numpy.ndarray, numpy.ndarray], float]): function that takes two arguments that are points in function landscape and calculates the distance between them.
        nl (float): Normalized range for neighborhood search :math:`\in (0, 1]`.
        F (float): Mutation parameter.
        CR (float): Crossover parameter :math:`\in [0, 1]`.
        Combination (Callable[numpy.ndarray, numpy.ndarray, numpy.ndarray, numpy.ndarray, float, float, float, float, float, float, Task, numpy.random.Generator]): Function for combining individuals to get new position/individual.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['AnarchicSocietyOptimization', 'ASO']

    @staticmethod
    def info():
        r"""Get basic information about the algorithm.

        Returns:
            str: Basic information.

        See Also:
            :func:`niapy.algorithms.algorithm.Algorithm.info`

        """
        return r"""Ahmadi-Javid, Amir. "Anarchic Society Optimization: A human-inspired method." Evolutionary Computation (CEC), 2011 IEEE Congress on. IEEE, 2011."""

    def __init__(self, population_size=43, alpha=(1, 0.83), gamma=(1.17, 0.56), theta=(0.932, 0.832), d=euclidean,
                 dn=euclidean, nl=1, mutation_rate=1.2, crossover_rate=0.25, combination=elitism, *args, **kwargs):
        r"""Initialize AnarchicSocietyOptimization.

        Args:
            population_size (Optional[int]): Population size.
            alpha (Optional[Tuple[float, ...]]): Factor for fickleness index function :math:`\in [0, 1]`.
            gamma (Optional[Tuple[float, ...]]): Factor for external irregularity index function :math:`\in [0, \infty)`.
            theta (Optional[List[float]]): Factor for internal irregularity index function :math:`\in [0, \infty)`.
            d (Optional[Callable[[float, float], float]]): function that takes two arguments that are function values and calculates the distance between them.
            dn (Optional[Callable[[numpy.ndarray, numpy.ndarray], float]]): function that takes two arguments that are points in function landscape and calculates the distance between them.
            nl (Optional[float]): Normalized range for neighborhood search :math:`\in (0, 1]`.
            mutation_rate (Optional[float]): Mutation parameter.
            crossover_rate (Optional[float]): Crossover parameter :math:`\in [0, 1]`.
            combination (Optional[Callable[numpy.ndarray, numpy.ndarray, numpy.ndarray, numpy.ndarray, float, float, float, float, float, float, Task, numpy.random.Generator]]): Function for combining individuals to get new position/individual.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        super().__init__(population_size, *args, **kwargs)
        self.alpha = alpha
        self.gamma = gamma
        self.theta = theta
        self.d = d
        self.dn = dn
        self.nl = nl
        self.mutation_rate = mutation_rate
        self.crossover_rate = crossover_rate
        self.Combination = combination

    def set_parameters(self, population_size=43, alpha=(1, 0.83), gamma=(1.17, 0.56), theta=(0.932, 0.832), d=euclidean,
                       dn=euclidean, nl=1, mutation_rate=1.2, crossover_rate=0.25, combination=elitism, **kwargs):
        r"""Set the parameters for the algorithm.

        Args:
            population_size (Optional[int]): Population size.
            alpha (Optional[Tuple[float, ...]]): Factor for fickleness index function :math:`\in [0, 1]`.
            gamma (Optional[Tuple[float, ...]]): Factor for external irregularity index function :math:`\in [0, \infty)`.
            theta (Optional[List[float]]): Factor for internal irregularity index function :math:`\in [0, \infty)`.
            d (Optional[Callable[[float, float], float]]): function that takes two arguments that are function values and calculates the distance between them.
            dn (Optional[Callable[[numpy.ndarray, numpy.ndarray], float]]): function that takes two arguments that are points in function landscape and calculates the distance between them.
            nl (Optional[float]): Normalized range for neighborhood search :math:`\in (0, 1]`.
            mutation_rate (Optional[float]): Mutation parameter.
            crossover_rate (Optional[float]): Crossover parameter :math:`\in [0, 1]`.
            combination (Optional[Callable[numpy.ndarray, numpy.ndarray, numpy.ndarray, numpy.ndarray, float, float, float, float, float, float, Task, numpy.random.Generator]]): Function for combining individuals to get new position/individual.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`
            * Combination methods:
                * :func:`niapy.algorithms.other.elitism`
                * :func:`niapy.algorithms.other.crossover`
                * :func:`niapy.algorithms.other.sequential`

        """
        super().set_parameters(population_size=population_size, **kwargs)
        self.alpha = alpha
        self.gamma = gamma
        self.theta = theta
        self.d = d
        self.dn = dn
        self.nl = nl
        self.mutation_rate = mutation_rate
        self.crossover_rate = crossover_rate
        self.Combination = combination

    def init(self, _task):
        r"""Initialize dynamic parameters of algorithm.

        Args:
            _task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray]
                1. Array of `self.alpha` propagated values
                2. Array of `self.gamma` propagated values
                3. Array of `self.theta` propagated values

        """
        return full_array(self.alpha, self.population_size), full_array(self.gamma, self.population_size), full_array(
            self.theta, self.population_size)

    @staticmethod
    def fickleness_index(x_f, xpb_f, xb_f, alpha):
        r"""Get fickleness index.

        Args:
            x_f (float): Individuals fitness/function value.
            xpb_f (float): Individuals personal best fitness/function value.
            xb_f (float): Current best found individuals fitness/function value.
            alpha (float): Fickleness factor.

        Returns:
            float: Fickleness index.

        """
        return 1 - alpha * xb_f / x_f - (1 - alpha) * xpb_f / x_f

    def external_irregularity(self, x_f, xnb_f, gamma):
        r"""Get external irregularity index.

        Args:
            x_f (float): Individuals fitness/function value.
            xnb_f (float): Individuals new fitness/function value.
            gamma (float): External irregularity factor.

        Returns:
            float: External irregularity index.

        """
        return 1 - np.exp(-gamma * self.d(x_f, xnb_f))

    def irregularity_index(self, x_f, xpb_f, theta):
        r"""Get internal irregularity index.

        Args:
            x_f (float): Individuals fitness/function value.
            xpb_f (float): Individuals personal best fitness/function value.
            theta (float): Internal irregularity factor.

        Returns:
            float: Internal irregularity index

        """
        return 1 - np.exp(-theta * self.d(x_f, xpb_f))

    def get_best_neighbors(self, i, population, population_fitness, rs):
        r"""Get neighbors of individual.

        Measurement of distance for neighborhood is defined with `self.nl`.
        Function for calculating distances is define with `self.dn`.

        Args:
            i (int): Index of individual for hum we are looking for neighbours.
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray[float]): Current population fitness/function values.
            rs (numpy.ndarray[float]): distance between individuals.

        Returns:
            numpy.ndarray[int]: Indexes that represent individuals closest to `i`-th individual.

        """
        nn = np.asarray([self.dn(population[i], population[j]) / rs for j in range(len(population))])
        return np.argmin(population_fitness[np.where(nn <= self.nl)])

    @staticmethod
    def update_personal_best(population, population_fitness, personal_best, personal_best_fitness):
        r"""Update personal best solution of all individuals in population.

        Args:
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray[float]): Current population fitness/function values.
            personal_best (numpy.ndarray): Current population best positions.
            personal_best_fitness (numpy.ndarray[float]): Current populations best positions fitness/function values.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float], numpy.ndarray, float]:
                1. New personal best positions for current population.
                2. New personal best positions function/fitness values for current population.
                3. New best individual.
                4. New best individual fitness/function value.

        """
        ix_pb = np.where(population_fitness < personal_best_fitness)
        personal_best[ix_pb], personal_best_fitness[ix_pb] = population[ix_pb], population_fitness[ix_pb]
        return personal_best, personal_best_fitness

    def init_population(self, task):
        r"""Initialize first population and additional arguments.

        Args:
            task (Task): Optimization task

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, dict]:
                1. Initialized population
                2. Initialized population fitness/function values
                3. Dict[str, Any]:
                    * x_best (numpy.ndarray): Initialized populations best positions.
                    * x_best_fitness (numpy.ndarray): Initialized populations best positions function/fitness values.
                    * alpha (numpy.ndarray):
                    * gamma (numpy.ndarray):
                    * theta (numpy.ndarray):
                    * rs (float): distance of search space.

        See Also:
            * :func:`niapy.algorithms.algorithm.Algorithm.init_population`
            * :func:`niapy.algorithms.other.aso.AnarchicSocietyOptimization.init`

        """
        population, population_fitness, d = Algorithm.init_population(self, task)
        alpha, gamma, theta = self.init(task)
        x_best, x_best_fitness = self.update_personal_best(population, task.optimization_type.value * population_fitness,
                                                           np.zeros((self.population_size, task.dimension)),
                                                           np.full(self.population_size, np.inf))
        d.update({'x_best': x_best, 'x_best_fitness': x_best_fitness, 'alpha': alpha, 'gamma': gamma, 'theta': theta,
                  'rs': self.d(task.upper, task.lower)})
        return population, population_fitness, d

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of AnarchicSocietyOptimization algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current populations positions.
            population_fitness (numpy.ndarray): Current populations function/fitness values.
            best_x (numpy.ndarray): Current global best individuals position.
            best_fitness (float): Current global best individual function/fitness value.
            **params: Additional arguments.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, dict]:
                1. Initialized population
                2. Initialized population fitness/function values
                3. New global best solution
                4. New global best solutions fitness/objective value
                5. Dict[str, Union[float, int, numpy.ndarray]:
                    * x_best (numpy.ndarray): Initialized populations best positions.
                    * x_best_fitness (numpy.ndarray): Initialized populations best positions function/fitness values.
                    * alpha (numpy.ndarray):
                    * gamma (numpy.ndarray):
                    * theta (numpy.ndarray):
                    * rs (float): distance of search space.

        """
        x_best = params.pop('x_best')
        x_best_fitness = params.pop('x_best_fitness')
        alpha = params.pop('alpha')
        gamma = params.pop('gamma')
        theta = params.pop('theta')
        rs = params.pop('rs')

        x_in = [self.get_best_neighbors(i, population, population_fitness, rs) for i in range(len(population))]
        mp_c, mp_s, mp_p = np.asarray(
            [self.fickleness_index(population_fitness[i], x_best_fitness[i], best_fitness, alpha[i]) for i in
             range(len(population))]), np.asarray(
            [self.external_irregularity(population_fitness[i], population_fitness[x_in[i]], gamma[i]) for i in
             range(len(population))]), np.asarray(
            [self.irregularity_index(population_fitness[i], x_best_fitness[i], theta[i]) for i in range(len(population))])
        x_tmp = np.asarray([self.Combination(population[i], x_best[i], best_x,
                                             population[self.integers(len(population), skip=[i])], mp_c[i], mp_s[i],
                                             mp_p[i], self.mutation_rate, self.crossover_rate, task, self.rng) for i in range(len(population))],
                           dtype=object)
        population, population_fitness = np.asarray([x_tmp[i][0] for i in range(len(population))]), np.asarray(
            [x_tmp[i][1] for i in range(len(population))])
        x_best, x_best_fitness = self.update_personal_best(population, population_fitness, x_best, x_best_fitness)
        best_x, best_fitness = self.get_best(population, population_fitness, best_x, best_fitness)
        return population, population_fitness, best_x, best_fitness, {'x_best': x_best,
                                                                      'x_best_fitness': x_best_fitness,
                                                                      'alpha': alpha,
                                                                      'gamma': gamma,
                                                                      'theta': theta,
                                                                      'rs': rs}



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')

__all__ = ['CoralReefsOptimization']


def default_sexual_crossover(pop, p, task, rng, **_kwargs):
    r"""Sexual reproduction of corals.

    Args:
        pop (numpy.ndarray): Current population.
        p (float): Probability in range [0, 1].
        task (Task): Optimization task.
        rng (numpy.random.Generator): Random generator.

    Returns:
        Tuple[numpy.ndarray, numpy.ndarray]:
            1. New population.
            2. New population function/fitness values.

    """
    for i in range(len(pop) // 2):
        pop[i] = np.asarray([pop[i, d] if rng.random() < p else pop[i * 2, d] for d in range(task.dimension)])
    return pop, np.apply_along_axis(task.eval, 1, pop)


def default_brooding(pop, p, task, rng, **_kwargs):
    r"""Brooding or internal sexual reproduction of corals.

    Args:
        pop (numpy.ndarray): Current population.
        p (float): Probability in range [0, 1].
        task (Task): Optimization task.
        rng (numpy.random.Generator): Random generator.

    Returns:
        Tuple[numpy.ndarray, numpy.ndarray]:
            1. New population.
            2. New population function/fitness values.

    """
    for i in range(len(pop)):
        pop[i] = task.repair(np.asarray(
            [pop[i, d] if rng.random() < p else task.lower[d] + task.range[d] * rng.random() for d in
             range(task.dimension)]), rng=rng)
    return pop, np.apply_along_axis(task.eval, 1, pop)


def move_corals(pop, p, f, task, rng, **_kwargs):
    r"""Move corals.

    Args:
        pop (numpy.ndarray): Current population.
        p (float): Probability in range [0, 1].
        f (float): Factor.
        task (Task): Optimization task.
        rng (numpy.random.Generator): Random generator.

    Returns:
        Tuple[numpy.ndarray, numpy.ndarray]:
            1. New population.
            2. New population function/fitness values.

    """
    for i in range(len(pop)):
        pop[i] = task.repair(
            np.asarray(
                [pop[i, d] if rng.random() < p else pop[i, d] + f * rng.random() for d in range(task.dimension)]),
            rng=rng)
    return pop, np.apply_along_axis(task.eval, 1, pop)


class CoralReefsOptimization(Algorithm):
    r"""Implementation of Coral Reefs Optimization Algorithm.

    Algorithm:
        Coral Reefs Optimization Algorithm

    Date:
        2018

    Authors:
        Klemen Berkovič

    License:
        MIT

    Reference Paper:
        S. Salcedo-Sanz, J. Del Ser, I. Landa-Torres, S. Gil-López, and J. A. Portilla-Figueras,
        “The Coral Reefs Optimization Algorithm: A Novel Metaheuristic for Efficiently Solving Optimization Problems,”
        The Scientific World Journal, vol. 2014, Article ID 739768, 15 pages, 2014.

    Reference URL:
        https://doi.org/10.1155/2014/739768

    Attributes:
        Name (List[str]): List of strings representing algorithm name.
        phi (float): Range of neighborhood.
        num_asexual_reproduction (int): Number of corals used in asexual reproduction.
        num_broadcast (int): Number of corals used in brooding.
        num_depredation (int): Number of corals used in depredation.
        k (int): Number of tries for larva setting.
        mutation_rate (float): Mutation variable :math:`\in [0, \infty]`.
        crossover_rate(float): Crossover rate in [0, 1].
        sexual_crossover (Callable[[numpy.ndarray, float, Task, numpy.random.Generator, Dict[str, Any]], Tuple[numpy.ndarray, numpy.ndarray[float]]]):
            Crossover function.
        brooding (Callable[[numpy.ndarray, float, Task, numpy.random.Generator, Dict[str, Any]], Tuple[numpy.ndarray, numpy.ndarray]]):
            Brooding function.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['CoralReefsOptimization', 'CRO']

    @staticmethod
    def info():
        r"""Get algorithms information.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""S. Salcedo-Sanz, J. Del Ser, I. Landa-Torres, S. Gil-López, and J. A. Portilla-Figueras,
        “The Coral Reefs Optimization Algorithm: A Novel Metaheuristic for Efficiently Solving Optimization Problems,”
        The Scientific World Journal, vol. 2014, Article ID 739768, 15 pages, 2014."""

    def __init__(self, population_size=25, phi=0.4, asexual_reproduction_prob=0.5, broadcast_prob=0.5,
                 depredation_prob=0.3, k=25, crossover_rate=0.5, mutation_rate=0.36,
                 sexual_crossover=default_sexual_crossover, brooding=default_brooding, *args, **kwargs):
        r"""Initialize CoralReefsOptimization.

        Args:
            population_size (int): population size for population initialization.
            phi (int): distance.
            asexual_reproduction_prob (float): Value $\in [0, 1]$ for Asexual reproduction size.
            broadcast_prob (float): Value $\in [0, 1]$ for brooding size.
            depredation_prob (float): Value $\in [0, 1]$ for Depredation size.
            k (int): Tries for larvae setting.
            sexual_crossover (Callable[[numpy.ndarray, float, Task, numpy.random.Generator, Dict[str, Any]], Tuple[numpy.ndarray, numpy.ndarray]]):
                Crossover function.
            crossover_rate (float): Crossover rate $\in [0, 1]$.
            brooding (Callable[[numpy.ndarray, float, Task, numpy.random.Generator, Dict[str, Any]], Tuple[numpy.ndarray, numpy.ndarray]]):
                brooding function.
            mutation_rate (float): Crossover rate $\in [0, 1]$.

        See Also:
            * :func:`niapy.algorithms.Algorithm.__init__`

        """
        super().__init__(population_size, *args, **kwargs)
        self.phi = phi
        self.k = k
        self.crossover_rate = crossover_rate
        self.mutation_rate = mutation_rate
        self.num_asexual_reproduction = int(self.population_size * asexual_reproduction_prob)
        self.num_broadcast = int(self.population_size * broadcast_prob)
        self.num_depredation = int(self.population_size * depredation_prob)
        self.sexual_crossover = sexual_crossover
        self.brooding = brooding

    def set_parameters(self, population_size=25, phi=0.4, asexual_reproduction_prob=0.5, broadcast_prob=0.5,
                       depredation_prob=0.3, k=25, crossover_rate=0.5, mutation_rate=0.36,
                       sexual_crossover=default_sexual_crossover, brooding=default_brooding, **kwargs):
        r"""Set the parameters of the algorithm.

        Args:
            population_size (int): population size for population initialization.
            phi (int): distance.
            asexual_reproduction_prob (float): Value $\in [0, 1]$ for Asexual reproduction size.
            broadcast_prob (float): Value $\in [0, 1]$ for brooding size.
            depredation_prob (float): Value $\in [0, 1]$ for Depredation size.
            k (int): Tries for larvae setting.
            sexual_crossover (Callable[[numpy.ndarray, float, Task, numpy.random.Generator, Dict[str, Any]], Tuple[numpy.ndarray, numpy.ndarray]]):
                Crossover function.
            crossover_rate (float): Crossover rate $\in [0, 1]$.
            brooding (Callable[[numpy.ndarray, float, Task, numpy.random.Generator, Dict[str, Any]], Tuple[numpy.ndarray, numpy.ndarray]]):
                brooding function.
            mutation_rate (float): Crossover rate $\in [0, 1]$.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        super().set_parameters(self, population_size=population_size, **kwargs)
        self.phi = phi
        self.k = k
        self.crossover_rate = crossover_rate
        self.mutation_rate = mutation_rate
        self.num_asexual_reproduction = int(self.population_size * asexual_reproduction_prob)
        self.num_broadcast = int(self.population_size * broadcast_prob)
        self.num_depredation = int(self.population_size * depredation_prob)
        self.sexual_crossover = sexual_crossover
        self.brooding = brooding

    def get_parameters(self):
        r"""Get parameters values of the algorithm.

        Returns:
            Dict[str, Any]: Algorithm parameters.

        """
        d = super().get_parameters()
        d.update({
            'phi': self.phi,
            'k': self.k,
            'crossover_rate': self.crossover_rate,
            'mutation_rate': self.mutation_rate,
            'num_asexual_reproduction': self.num_asexual_reproduction,
            'num_depredation': self.num_depredation,
            'num_broadcast': self.num_broadcast
        })
        return d

    def asexual_reproduction(self, reef, reef_fitness, best_x, best_fitness, task):
        r"""Asexual reproduction of corals.

        Args:
            reef (numpy.ndarray): Current population of reefs.
            reef_fitness (numpy.ndarray): Current populations function/fitness values.
            best_x (numpy.ndarray): Global best coordinates.
            best_fitness (float): Global best fitness.
            task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray]:
                1. New population.
                2. New population fitness/function values.

        See Also:
            * :func:`niapy.algorithms.basic.CoralReefsOptimization.setting`
            * :func:`niapy.algorithms.basic.default_brooding`

        """
        brooding_indices = np.argsort(reef_fitness)[:self.num_asexual_reproduction]
        new_reef, new_reef_fitness = self.brooding(reef[brooding_indices], self.mutation_rate, task, rng=self.rng)
        best_x, best_fitness = self.get_best(new_reef, new_reef_fitness, best_x, best_fitness)
        reef, reef_fitness, best_x, best_fitness = self.settling(reef, reef_fitness, new_reef, new_reef_fitness, best_x,
                                                                 best_fitness, task)
        return reef, reef_fitness, best_x, best_fitness

    def depredation(self, reef, reef_fitness):
        r"""Depredation operator for reefs.

        Args:
            reef (numpy.ndarray): Current reefs.
            reef_fitness (numpy.ndarray): Current reefs function/fitness values.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray]:
                1. Best individual
                2. Best individual fitness/function value

        """
        depredation_index = np.argsort(reef_fitness)[::-1][:self.num_depredation]
        return np.delete(reef, depredation_index), np.delete(reef_fitness, depredation_index)

    def settling(self, reef, reef_fitness, new_reef, new_reef_fitness, best_x, best_fitness, task):
        r"""Operator for setting reefs.

        New reefs try to settle to selected position in search space.
        New reefs are successful if their fitness values is better or if they have no reef occupying same search space.

        Args:
            reef (numpy.ndarray): Current population of reefs.
            reef_fitness (numpy.ndarray): Current populations function/fitness values.
            new_reef (numpy.ndarray): New population of reefs.
            new_reef_fitness (numpy.ndarray): New populations function/fitness values.
            best_x (numpy.ndarray): Global best solution.
            best_fitness (float): Global best solutions fitness/objective value.
            task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float]:
                1. New settled population.
                2. New settled population fitness/function values.

        """
        def update(arr, phi, xb, fxb):
            dist = np.asarray([np.sqrt(np.sum((arr - e) ** 2, axis=1)) for e in new_reef])
            ind = np.unique(np.where(dist < phi)[0])
            if ind.any():
                new_reef[ind], new_reef_fitness[ind] = move_corals(new_reef[ind],
                                                                   self.mutation_rate,
                                                                   self.mutation_rate,
                                                                   task,
                                                                   rng=self.rng)
                xb, fxb = self.get_best(new_reef[ind], new_reef_fitness[ind], xb, fxb)
            return xb, fxb

        for i in range(self.k):
            best_x, best_fitness = update(reef, self.phi, best_x, best_fitness)
            best_x, best_fitness = update(new_reef, self.phi, best_x, best_fitness)
        distances = np.asarray([np.sqrt(np.sum((reef - e) ** 2, axis=1)) for e in new_reef])
        indices = np.unique(np.where(distances >= self.phi)[0])
        return np.append(reef, new_reef[indices], 0), np.append(reef_fitness, new_reef_fitness[indices], 0), best_x, best_fitness

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of Coral Reefs Optimization algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray): Current population fitness/function value.
            best_x (numpy.ndarray): Global best solution.
            best_fitness (float): Global best solution fitness/function value.
            **params: Additional arguments

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. New global best solution
                4. New global best solutions fitness/objective value
                5. Additional arguments:

        See Also:
            * :func:`niapy.algorithms.basic.CoralReefsOptimization.sexual_crossover`
            * :func:`niapy.algorithms.basic.CoralReefsOptimization.brooding`

        """
        broadcast_index = self.rng.choice(len(population), size=self.num_broadcast, replace=False)
        reef_broadcast, reef_broadcast_fitness = self.sexual_crossover(population[broadcast_index],
                                                                       self.crossover_rate,
                                                                       task,
                                                                       rng=self.rng)
        best_x, best_fitness = self.get_best(reef_broadcast, reef_broadcast_fitness, best_x, best_fitness)
        reef_brooding, reef_brooding_fitness = self.brooding(np.delete(population, broadcast_index, 0),
                                                             self.mutation_rate,
                                                             task,
                                                             rng=self.rng)
        best_x, best_fitness = self.get_best(reef_brooding, reef_brooding_fitness, best_x, best_fitness)
        new_reef, new_reef_fitness, best_x, best_fitness = self.settling(population,
                                                                         population_fitness,
                                                                         np.append(reef_broadcast, reef_brooding, 0),
                                                                         np.append(reef_broadcast_fitness,
                                                                                   reef_brooding_fitness, 0), best_x,
                                                                         best_fitness,
                                                                         task)
        population, population_fitness, best_x, best_fitness = self.asexual_reproduction(new_reef,
                                                                                         new_reef_fitness,
                                                                                         best_x,
                                                                                         best_fitness,
                                                                                         task)
        if (task.iters + 1) % self.k == 0:
            population, population_fitness = self.depredation(population, population_fitness)
        return population, population_fitness, best_x, best_fitness, {}



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')

__all__ = ['BatAlgorithm']


class BatAlgorithm(Algorithm):
    r"""Implementation of Bat algorithm.

    Algorithm:
        Bat algorithm

    Date:
        2015

    Authors:
        Iztok Fister Jr., Marko Burjek and Klemen Berkovič

    License:
        MIT

    Reference paper:
        Yang, Xin-She. "A new metaheuristic bat-inspired algorithm." Nature inspired cooperative strategies for optimization (NICSO 2010). Springer, Berlin, Heidelberg, 2010. 65-74.

    Attributes:
        Name (List[str]): List of strings representing algorithm name.
        loudness (float): Initial loudness.
        pulse_rate (float): Initial pulse rate.
        alpha (float): Parameter for controlling loudness decrease.
        gamma (float): Parameter for controlling pulse rate increase.
        min_frequency (float): Minimum frequency.
        max_frequency (float): Maximum frequency.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['BatAlgorithm', 'BA']

    @staticmethod
    def info():
        r"""Get algorithms information.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""Yang, Xin-She. "A new metaheuristic bat-inspired algorithm." Nature inspired cooperative strategies for optimization (NICSO 2010). Springer, Berlin, Heidelberg, 2010. 65-74."""

    def __init__(self, population_size=40, loudness=1.0, pulse_rate=1.0, alpha=0.97, gamma=0.1, min_frequency=0.0,
                 max_frequency=2.0, *args, **kwargs):
        """Initialize BatAlgorithm.

        Args:
            population_size (Optional[int]): Population size.
            loudness (Optional[float]): Initial loudness.
            pulse_rate (Optional[float]): Initial pulse rate.
            alpha (Optional[float]): Parameter for controlling loudness decrease.
            gamma (Optional[float]): Parameter for controlling pulse rate increase.
            min_frequency (Optional[float]): Minimum frequency.
            max_frequency (Optional[float]): Maximum frequency.

        See Also:
            :func:`niapy.algorithms.Algorithm.__init__`

        """
        super().__init__(population_size, *args, **kwargs)
        self.loudness = loudness
        self.pulse_rate = pulse_rate
        self.alpha = alpha
        self.gamma = gamma
        self.min_frequency = min_frequency
        self.max_frequency = max_frequency

    def set_parameters(self, population_size=20, loudness=1.0, pulse_rate=1.0, alpha=0.97, gamma=0.1, min_frequency=0.0,
                       max_frequency=2.0, **kwargs):
        r"""Set the parameters of the algorithm.

        Args:
            population_size (Optional[int]): Population size.
            loudness (Optional[float]): Initial loudness.
            pulse_rate (Optional[float]): Initial pulse rate.
            alpha (Optional[float]): Parameter for controlling loudness decrease.
            gamma (Optional[float]): Parameter for controlling pulse rate increase.
            min_frequency (Optional[float]): Minimum frequency.
            max_frequency (Optional[float]): Maximum frequency.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        super().set_parameters(population_size=population_size, **kwargs)
        self.loudness = loudness
        self.pulse_rate = pulse_rate
        self.alpha = alpha
        self.gamma = gamma
        self.min_frequency = min_frequency
        self.max_frequency = max_frequency

    def get_parameters(self):
        r"""Get parameters of the algorithm.

        Returns:
            Dict[str, Any]: Algorithm parameters.

        """
        d = super().get_parameters()
        d.update({
            'loudness': self.loudness,
            'pulse_rate': self.pulse_rate,
            'alpha': self.alpha,
            'gamma': self.gamma,
            'min_frequency': self.min_frequency,
            'max_frequency': self.max_frequency
        })
        return d

    def init_population(self, task):
        r"""Initialize the starting population.

        Parameters:
            task (Task): Optimization task

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float], Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. Additional arguments:
                    * velocities (numpy.ndarray[float]): Velocities.
                    * alpha (float): Previous iterations loudness.

        See Also:
            * :func:`niapy.algorithms.Algorithm.init_population`

        """
        population, fitness, d = super().init_population(task)
        velocities = np.zeros((self.population_size, task.dimension))
        d.update({'velocities': velocities, 'loudness': self.loudness})
        return population, fitness, d

    def local_search(self, best, loudness, task, **kwargs):
        r"""Improve the best solution according to the Yang (2010).

        Args:
            best (numpy.ndarray): Global best individual.
            loudness (float): Current loudness.
            task (Task): Optimization task.

        Returns:
            numpy.ndarray: New solution based on global best individual.

        """
        return task.repair(best + 0.1 * self.standard_normal(task.dimension) * loudness)

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of Bat Algorithm.

        Parameters:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population
            population_fitness (numpy.ndarray[float]): Current population fitness/function values
            best_x (numpy.ndarray): Current best individual
            best_fitness (float): Current best individual function/fitness value
            params (Dict[str, Any]): Additional algorithm arguments

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population
                2. New population fitness/function values
                3. New global best solution
                4. New global best fitness/objective value
                5. Additional arguments:
                    * velocities (numpy.ndarray): Velocities.
                    * alpha (float): Previous iterations loudness.

        """
        velocities = params.pop('velocities')
        loudness = params.pop('loudness') * self.alpha

        pulse_rate = self.pulse_rate * (1 - np.exp(-self.gamma * task.iters))

        for i in range(self.population_size):
            frequency = self.min_frequency + (self.max_frequency - self.min_frequency) * self.random()
            velocities[i] += (population[i] - best_x) * frequency
            if self.random() < pulse_rate:
                solution = self.local_search(best=best_x, loudness=loudness, task=task, i=i, population=population)
            else:
                solution = task.repair(population[i] + velocities[i], rng=self.rng)
            new_fitness = task.eval(solution)
            if (new_fitness <= population_fitness[i]) and (self.random() > loudness):
                population[i], population_fitness[i] = solution, new_fitness
            if new_fitness <= best_fitness:
                best_x, best_fitness = solution.copy(), new_fitness
        return population, population_fitness, best_x, best_fitness, {'velocities': velocities, 'loudness': loudness}



# encoding=utf8

"""Implementations of Powell function."""
import numpy as np

from niapy.problems.problem import Problem

__all__ = ['Powell']


class Powell(Problem):
    r"""Implementations of Powell functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Powell Function**

        :math:`f(\textbf{x}) = \sum_{i = 1}^{D / 4} \left( (x_{4 i - 3} + 10 x_{4 i - 2})^2 + 5 (x_{4 i - 1} - x_{4 i})^2 + (x_{4 i - 2} - 2 x_{4 i - 1})^4 + 10 (x_{4 i - 3} - x_{4 i})^4 \right)`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-4, 5]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        :math:`f(\textbf{x}^*) = 0` at :math:`\textbf{x}^* = (0, \cdots, 0)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = \sum_{i = 1}^{D / 4} \left( (x_{4 i - 3} + 10 x_{4 i - 2})^2 + 5 (x_{4 i - 1} - x_{4 i})^2 + (x_{4 i - 2} - 2 x_{4 i - 1})^4 + 10 (x_{4 i - 3} - x_{4 i})^4 \right)$

        Equation:
            \begin{equation} f(\textbf{x}) = \sum_{i = 1}^{D / 4} \left( (x_{4 i - 3} + 10 x_{4 i - 2})^2 + 5 (x_{4 i - 1} - x_{4 i})^2 + (x_{4 i - 2} - 2 x_{4 i - 1})^4 + 10 (x_{4 i - 3} - x_{4 i})^4 \right) \end{equation}

        Domain:
            $-4 \leq x_i \leq 5$

    Reference:
        https://www.sfu.ca/~ssurjano/powell.html

    """

    def __init__(self, dimension=4, lower=-4.0, upper=5.0, *args, **kwargs):
        r"""Initialize Powell problem..

        Args:
            dimension (Optional[int]): Dimension of the problem.
            lower (Optional[Union[float, Iterable[float]]]): Lower bounds of the problem.
            upper (Optional[Union[float, Iterable[float]]]): Upper bounds of the problem.

        See Also:
            :func:`niapy.problems.Problem.__init__`

        """
        super().__init__(dimension, lower, upper, *args, **kwargs)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = \sum_{i = 1}^{D / 4} \left( (x_{4 i - 3} + 10 x_{4 i - 2})^2 + 5 (x_{4 i - 1} - x_{4 i})^2 + (x_{4 i - 2} - 2 x_{4 i - 1})^4 + 10 (x_{4 i - 3} - x_{4 i})^4 \right)$'''

    def _evaluate(self, x):
        x1 = x[range(1, self.dimension - 3, 4)]
        x2 = x[range(2, self.dimension - 2, 4)]
        x3 = x[range(3, self.dimension - 1, 4)]
        x4 = x[range(4, self.dimension, 4)]

        term1 = (x1 + 10 * x2) ** 2.0
        term2 = 5 * (x3 - x4) ** 2.0
        term3 = (x2 - 2 * x3) ** 4.0
        term4 = 10 * (x1 - x4) ** 4.0
        return np.sum(term1 + term2 + term3 + term4)



# encoding=utf8

"""Argparser class."""

import logging
import sys
from argparse import ArgumentParser

import numpy as np

import niapy.problems as problems
from niapy.task import OptimizationType

logging.basicConfig()
logger = logging.getLogger('niapy.util.argparse')
logger.setLevel('INFO')

__all__ = ['get_argparser', 'get_args', 'get_args_dict']


def _get_problem_names():
    r"""Get problem names."""
    return problems.__all__


def _optimization_type(x):
    r"""Get OptimizationType from string.

    Args:
        x (str): String representing optimization type.

    Returns:
        OptimizationType: Optimization type based on type that is defined as enum.

    """
    if x not in ['min', 'max']:
        logger.info('You can use only [min, max], using min')
    return OptimizationType.MAXIMIZATION if x == 'max' else OptimizationType.MINIMIZATION


def get_argparser():
    r"""Create/Make parser for parsing string.

    Parser:
        * `-a` or `--algorithm` (str):
            Name of algorithm to use. Default value is `jDE`.
        * `-p` or `--problem` (str):
            Name of problem to use. Default values is `Ackley`.
        * `-d` or `--dimension` (int):
            Number of dimensions/components used by problem. Default values is `10`.
        * `--max-evals` (int):
            Number of maximum function evaluations. Default values is `inf`.
        * `--max-iters` (int):
            Number of maximum algorithm iterations/generations. Default values is `inf`.
        * `-n` or  `--population-size` (int):
            Number of individuals in population. Default values is `43`.
        * `-r` or `--run-type` (str);
            Run type of run. Value can be:
                * '': No output during the run. Output is shown only at the end of algorithm run.
                * `log`: Output is shown every time new global best solution is found
                * `plot`: Output is shown only at the end of run. Output is shown as graph plotted in matplotlib. Graph represents convergence of algorithm over run time of algorithm.

            Default value is `''`.
        * `--seed` (list of int or int):
            Set the starting seed of algorithm run. If multiple runs, user can provide list of ints, where each int usd use at new run. Default values is `None`.
        * `--opt-type` (str):
            Optimization type of the run. Values can be:
                * `min`: For minimization problems
                * `max`: For maximization problems

            Default value is `min`.

    Returns:
        ArgumentParser: Parser for parsing arguments from string.

    See Also:
        * :class:`ArgumentParser`
        * :func:`ArgumentParser.add_argument`

    """
    parser, problem_names = ArgumentParser(description='Runner example.'), _get_problem_names()
    parser.add_argument('-a', '--algorithm', dest='algo', default='jDE', type=str)
    parser.add_argument('-p', '--problem', dest='problem', nargs='*', default=problem_names[0], choices=problem_names, type=str)
    parser.add_argument('-d', '--dimension', dest='dimension', default=10, type=int)
    parser.add_argument('--max-evals', dest='max_evals', default=np.inf, type=int)
    parser.add_argument('--max-iters', dest='max_iters', default=np.inf, type=int)
    parser.add_argument('-n', '--population-size', dest='population_size', default=43, type=int)
    parser.add_argument('-r', '--run-type', dest='run_type', choices=['', 'log', 'plot'], default='', type=str)
    parser.add_argument('--seed', dest='seed', nargs='+', default=[None], type=int)
    parser.add_argument('--opt-type', dest='opt_type', default=_optimization_type('min'), type=_optimization_type)
    return parser


def get_args(argv):
    r"""Parse arguments form input string.

    Args:
        argv (List[str]): List to parse.

    Returns:
        Dict[str, Union[float, int, str, OptimizationType]]: Where key represents argument name and values it's value.

    See Also:
        * :func:`niapy.util.argparser.get_argparser`.
        * :func:`ArgumentParser.parse_args`

    """
    parser = get_argparser()
    a = parser.parse_args(argv)
    return a


def get_args_dict(argv):
    r"""Parse input string.

    Args:
        argv (List[str]): Input string to parse for arguments

    Returns:
        dict: Parsed input string

    See Also:
        * :func:`niapy.utils.get_args`

    """
    return vars(get_args(argv))


if __name__ == '__main__':
    r"""Run the algorithms based on parameters from the command line interface."""
    args = get_args(sys.argv[1:])
    logger.info(str(args))



# encoding=utf8
import copy
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm, Individual, default_individual_init

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')

__all__ = ['ArtificialBeeColonyAlgorithm']


class SolutionABC(Individual):
    r"""Representation of solution for Artificial Bee Colony Algorithm.

    Date:
        2018

    Author:
        Klemen Berkovič

    See Also:
        * :class:`niapy.algorithms.Individual`

    """

    pass


class ArtificialBeeColonyAlgorithm(Algorithm):
    r"""Implementation of Artificial Bee Colony algorithm.

    Algorithm:
        Artificial Bee Colony algorithm

    Date:
        2018

    Author:
        Uros Mlakar and Klemen Berkovič

    License:
        MIT

    Reference paper:
        Karaboga, D., and Bahriye B. "A powerful and efficient algorithm for numerical function optimization: artificial bee colony (ABC) algorithm." Journal of global optimization 39.3 (2007): 459-471.

    Arguments
        Name (List[str]): List containing strings that represent algorithm names
        limit (Union[float, numpy.ndarray[float]]): Maximum number of cycles without improvement.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['ArtificialBeeColonyAlgorithm', 'ABC']

    @staticmethod
    def info():
        r"""Get algorithms information.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""Karaboga, D., and Bahriye B. "A powerful and efficient algorithm for numerical function optimization: artificial bee colony (ABC) algorithm." Journal of global optimization 39.3 (2007): 459-471."""

    def __init__(self, population_size=10, limit=100, *args, **kwargs):
        """Initialize ArtificialBeeColonyAlgorithm.

        Args:
            population_size (Optional[int]): Population size.
            limit (Optional[int]): Maximum number of cycles without improvement.

        See Also:
            :func:`niapy.algorithms.Algorithm.__init__`

        """
        super().__init__(population_size, initialization_function=default_individual_init, individual_type=SolutionABC,
                         *args, **kwargs)
        self.limit = limit
        self.food_number = self.population_size // 2

    def set_parameters(self, population_size=10, limit=100, **kwargs):
        r"""Set the parameters of Artificial Bee Colony Algorithm.

        Args:
            population_size(Optional[int]): Population size.
            limit (Optional[int]): Maximum number of cycles without improvement.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        super().set_parameters(population_size=population_size, initialization_function=default_individual_init,
                               individual_type=SolutionABC, **kwargs)
        self.food_number = self.population_size // 2
        self.limit = limit

    def get_parameters(self):
        """Get parameters."""
        params = super().get_parameters()
        params.update({
            'limit': self.limit
        })
        return params

    def calculate_probabilities(self, foods):
        r"""Calculate the probes.

        Args:
            foods (numpy.ndarray): Current population.

        Returns:
            numpy.ndarray: Probabilities.

        """
        probs = np.asarray([1.0 / (foods[i].f + 0.01) for i in range(self.food_number)])
        return probs / np.sum(probs)

    def init_population(self, task):
        r"""Initialize the starting population.

        Args:
            task (Task): Optimization task

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float], Dict[str, Any]]:
                1. New population
                2. New population fitness/function values
                3. Additional arguments:
                    * trials (numpy.ndarray): Number of cycles without improvement.

        See Also:
            * :func:`niapy.algorithms.Algorithm.init_population`

        """
        foods, fpop, _ = super().init_population(task)
        trials = np.zeros(self.food_number, dtype=np.int32)
        return foods, fpop, {'trials': trials}

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of  the algorithm.

        Args:
            task (Task): Optimization task
            population (numpy.ndarray): Current population
            population_fitness (numpy.ndarray[float]): Function/fitness values of current population
            best_x (numpy.ndarray): Current best individual
            best_fitness (float): Current best individual fitness/function value
            params (Dict[str, Any]): Additional parameters

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population
                2. New population fitness/function values
                3. New global best solution
                4. New global best fitness/objective value
                5. Additional arguments:
                    * trials (numpy.ndarray): Number of cycles without improvement.

        """
        trials = params.pop('trials')

        for i in range(self.food_number):
            new_solution = copy.deepcopy(population[i])
            param2change = int(self.random() * task.dimension)
            neighbor = int(self.food_number * self.random())
            new_solution.x[param2change] = population[i].x[param2change] + self.uniform(-1, 1) * (
                    population[i].x[param2change] - population[neighbor].x[param2change])
            new_solution.evaluate(task, rng=self.rng)
            if new_solution.f < population[i].f:
                population[i] = new_solution
                trials[i] = 0
                if new_solution.f < best_fitness:
                    best_x = new_solution.x.copy()
                    best_fitness = new_solution.f
            else:
                trials[i] += 1
        probabilities, t, s = self.calculate_probabilities(population), 0, 0
        while t < self.food_number:
            if self.random() < probabilities[s]:
                t += 1
                solution = copy.deepcopy(population[s])
                param2change = int(self.random() * task.dimension)
                neighbor = int(self.food_number * self.random())
                while neighbor == s:
                    neighbor = int(self.food_number * self.random())
                solution.x[param2change] = population[s].x[param2change] + self.uniform(-1, 1) * (
                        population[s].x[param2change] - population[neighbor].x[param2change])
                solution.evaluate(task, rng=self.rng)
                if solution.f < population[s].f:
                    population[s] = solution
                    trials[s] = 0
                    if solution.f < best_fitness:
                        best_x = solution.x.copy()
                        best_fitness = solution.f
                else:
                    trials[s] += 1
            s += 1
            if s == self.food_number:
                s = 0
        mi = np.argmax(trials)
        if trials[mi] >= self.limit:
            population[mi], trials[mi] = SolutionABC(task=task, rng=self.rng), 0
            if population[mi].f < best_fitness:
                best_x, best_fitness = population[mi].x.copy(), population[mi].f
        return population, np.asarray([f.f for f in population]), best_x, best_fitness, {'trials': trials}



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.modified')
logger.setLevel('INFO')

__all__ = ['AdaptiveBatAlgorithm', 'SelfAdaptiveBatAlgorithm']


class AdaptiveBatAlgorithm(Algorithm):
    r"""Implementation of Adaptive bat algorithm.

    Algorithm:
        Adaptive bat algorithm

    Date:
        April 2019

    Authors:
        Klemen Berkovič

    License:
        MIT

    Attributes:
        Name (List[str]): List of strings representing algorithm name.
        epsilon (float): Scaling factor.
        alpha (float): Constant for updating loudness.
        pulse_rate (float): Pulse rate.
        min_frequency (float): Minimum frequency.
        max_frequency (float): Maximum frequency.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['AdaptiveBatAlgorithm', 'ABA']

    @staticmethod
    def info():
        r"""Get basic information about the algorithm.

        Returns:
            str: Basic information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""TODO"""

    def __init__(self, population_size=100, starting_loudness=0.5, epsilon=0.001, alpha=1.0, pulse_rate=0.5,
                 min_frequency=0.0, max_frequency=2.0, *args, **kwargs):
        """Initialize AdaptiveBatAlgorithm.

        Args:
            population_size (Optional[int]): Population size.
            starting_loudness (Optional[float]): Starting loudness.
            epsilon (Optional[float]): Scaling factor.
            alpha (Optional[float]): Constant for updating loudness.
            pulse_rate (Optional[float]): Pulse rate.
            min_frequency (Optional[float]): Minimum frequency.
            max_frequency (Optional[float]): Maximum frequency.

        See Also:
            * :func:`niapy.algorithms.Algorithm.__init__`

        """
        super().__init__(population_size, *args, **kwargs)
        self.starting_loudness = starting_loudness
        self.epsilon = epsilon
        self.alpha = alpha
        self.pulse_rate = pulse_rate
        self.min_frequency = min_frequency
        self.max_frequency = max_frequency

    def set_parameters(self, population_size=100, starting_loudness=0.5, epsilon=0.001, alpha=1.0, pulse_rate=0.5,
                       min_frequency=0.0, max_frequency=2.0, **kwargs):
        r"""Set the parameters of the algorithm.

        Args:
            population_size (Optional[int]): Population size.
            starting_loudness (Optional[float]): Starting loudness.
            epsilon (Optional[float]): Scaling factor.
            alpha (Optional[float]): Constant for updating loudness.
            pulse_rate (Optional[float]): Pulse rate.
            min_frequency (Optional[float]): Minimum frequency.
            max_frequency (Optional[float]): Maximum frequency.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        super().set_parameters(population_size=population_size, **kwargs)
        self.starting_loudness = starting_loudness
        self.epsilon = epsilon
        self.alpha = alpha
        self.pulse_rate = pulse_rate
        self.min_frequency = min_frequency
        self.max_frequency = max_frequency

    def get_parameters(self):
        r"""Get algorithm parameters.

        Returns:
            Dict[str, Any]: Arguments values.

        See Also:
            * :func:`niapy.algorithms.algorithm.Algorithm.get_parameters`

        """
        d = super().get_parameters()
        d.update({
            'starting_loudness': self.starting_loudness,
            'epsilon': self.epsilon,
            'alpha': self.alpha,
            'pulse_rate': self.pulse_rate,
            'min_frequency': self.min_frequency,
            'max_frequency': self.max_frequency
        })
        return d

    def init_population(self, task):
        r"""Initialize the starting population.

        Args:
            task (Task): Optimization task

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float], Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. Additional arguments:
                    * loudness (float): Loudness.
                    * velocities (numpy.ndarray[float]): Velocity.

        See Also:
            * :func:`niapy.algorithms.Algorithm.init_population`

        """
        population, fitness, d = super().init_population(task)
        loudness = np.full(self.population_size, self.starting_loudness)
        velocities = np.zeros((self.population_size, task.dimension))
        d.update({'loudness': loudness, 'velocities': velocities})
        return population, fitness, d

    def local_search(self, best, loudness, task, **kwargs):
        r"""Improve the best solution according to the Yang (2010).

        Args:
            best (numpy.ndarray): Global best individual.
            loudness (float): Loudness.
            task (Task): Optimization task.

        Returns:
            numpy.ndarray: New solution based on global best individual.

        """
        return task.repair(best + self.epsilon * loudness * self.normal(0, 1, task.dimension), rng=self.rng)

    def update_loudness(self, loudness):
        r"""Update loudness when the prey is found.

        Args:
            loudness (float): Loudness.

        Returns:
            float: New loudness.

        """
        new_loudness = loudness * self.alpha
        return new_loudness if new_loudness > 1e-13 else self.starting_loudness

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of Bat Algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population
            population_fitness (numpy.ndarray[float]): Current population fitness/function values
            best_x (numpy.ndarray): Current best individual
            best_fitness (float): Current best individual function/fitness value
            params (Dict[str, Any]): Additional algorithm arguments

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float], Dict[str, Any]]:
                1. New population
                2. New population fitness/function values
                3. Additional arguments:
                    * loudness (numpy.ndarray[float]): Loudness.
                    * velocities (numpy.ndarray[float]): Velocities.

        """
        loudness = params.pop('loudness')
        velocities = params.pop('velocities')

        for i in range(self.population_size):
            frequency = self.min_frequency + (self.max_frequency - self.min_frequency) * self.random()
            velocities[i] += (population[i] - best_x) * frequency
            if self.random() > self.pulse_rate:
                solution = self.local_search(best=best_x, loudness=loudness[i], task=task, i=i, Sol=population)
            else:
                solution = task.repair(population[i] + velocities[i], rng=self.rng)
            new_fitness = task.eval(solution)
            if (new_fitness <= population_fitness[i]) and (self.random() < loudness[i]):
                population[i], population_fitness[i] = solution, new_fitness
            if new_fitness <= best_fitness:
                best_x, best_fitness, loudness[i] = solution.copy(), new_fitness, self.update_loudness(loudness[i])
        return population, population_fitness, best_x, best_fitness, {'loudness': loudness, 'velocities': velocities}


class SelfAdaptiveBatAlgorithm(AdaptiveBatAlgorithm):
    r"""Implementation of Hybrid bat algorithm.

    Algorithm:
        Self Adaptive Bat Algorithm

    Date:
        April 2019

    Author:
        Klemen Berkovič

    License:
        MIT

    Reference paper:
        Fister Jr., Iztok and Fister, Dusan and Yang, Xin-She. "A Hybrid Bat Algorithm". Elektrotehniški vestnik, 2013. 1-7.

    Attributes:
        Name (List[str]): List of strings representing algorithm name.
        A_l (Optional[float]): Lower limit of loudness.
        A_u (Optional[float]): Upper limit of loudness.
        r_l (Optional[float]): Lower limit of pulse rate.
        r_u (Optional[float]): Upper limit of pulse rate.
        tao_1 (Optional[float]): Learning rate for loudness.
        tao_2 (Optional[float]): Learning rate for pulse rate.

    See Also:
        * :class:`niapy.algorithms.basic.BatAlgorithm`

    """

    Name = ['SelfAdaptiveBatAlgorithm', 'SABA']

    @staticmethod
    def info():
        r"""Get basic information about the algorithm.

        Returns:
            str: Basic information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""Fister Jr., Iztok and Fister, Dusan and Yang, Xin-She. "A Hybrid Bat Algorithm". Elektrotehniški vestnik, 2013. 1-7."""

    def __init__(self, min_loudness=0.9, max_loudness=1.0, min_pulse_rate=0.001, max_pulse_rate=0.1, tao_1=0.1,
                 tao_2=0.1, *args, **kwargs):
        """Initialize SelfAdaptiveBatAlgorithm.

        Args:
            min_loudness (Optional[float]): Lower limit of loudness.
            max_loudness (Optional[float]): Upper limit of loudness.
            min_pulse_rate (Optional[float]): Lower limit of pulse rate.
            max_pulse_rate (Optional[float]): Upper limit of pulse rate.
            tao_1 (Optional[float]): Learning rate for loudness.
            tao_2 (Optional[float]): Learning rate for pulse rate.

        See Also:
            * :func:`niapy.algorithms.modified.AdaptiveBatAlgorithm.__init__`

        """
        super().__init__(*args, **kwargs)
        self.min_loudness = min_loudness
        self.max_loudness = max_loudness
        self.min_pulse_rate = min_pulse_rate
        self.max_pulse_rate = max_pulse_rate
        self.tao_1 = tao_1
        self.tao_2 = tao_2

    def set_parameters(self, min_loudness=0.9, max_loudness=1.0, min_pulse_rate=0.001, max_pulse_rate=0.1, tao_1=0.1, tao_2=0.1, **kwargs):
        r"""Set core parameters of HybridBatAlgorithm algorithm.

        Args:
            min_loudness (Optional[float]): Lower limit of loudness.
            max_loudness (Optional[float]): Upper limit of loudness.
            min_pulse_rate (Optional[float]): Lower limit of pulse rate.
            max_pulse_rate (Optional[float]): Upper limit of pulse rate.
            tao_1 (Optional[float]): Learning rate for loudness.
            tao_2 (Optional[float]): Learning rate for pulse rate.

        See Also:
            * :func:`niapy.algorithms.modified.AdaptiveBatAlgorithm.set_parameters`

        """
        super().set_parameters(**kwargs)
        self.min_loudness = min_loudness
        self.max_loudness = max_loudness
        self.min_pulse_rate = min_pulse_rate
        self.max_pulse_rate = max_pulse_rate
        self.tao_1 = tao_1
        self.tao_2 = tao_2

    def get_parameters(self):
        r"""Get parameters of the algorithm.

        Returns:
            Dict[str, Any]: Parameters of the algorithm.

        See Also:
            * :func:`niapy.algorithms.modified.AdaptiveBatAlgorithm.get_parameters`

        """
        d = AdaptiveBatAlgorithm.get_parameters(self)
        d.update({
            'min_loudness': self.min_loudness,
            'max_loudness': self.max_loudness,
            'min_pulse_rate': self.min_pulse_rate,
            'max_pulse_rate': self.max_pulse_rate,
            'tao_1': self.tao_1,
            'tao_2': self.tao_2
        })
        return d

    def init_population(self, task):
        population, fitness, d = super().init_population(task)
        pulse_rates = np.full(self.population_size, self.pulse_rate)
        d.update({'pulse_rates': pulse_rates})
        return population, fitness, d

    def self_adaptation(self, loudness, pulse_rate):
        r"""Adaptation step.

        Args:
            loudness (float): Current loudness.
            pulse_rate (float): Current pulse rate.

        Returns:
            Tuple[float, float]:
                1. New loudness.
                2. Nwq pulse rate.

        """
        return self.min_loudness + self.random() * (
                self.max_loudness - self.min_loudness) if self.random() < self.tao_1 else loudness, self.min_pulse_rate + self.random() * (
                self.max_pulse_rate - self.min_pulse_rate) if self.random() < self.tao_2 else pulse_rate

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of Bat Algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population
            population_fitness (numpy.ndarray[float]): Current population fitness/function values
            best_x (numpy.ndarray): Current best individual
            best_fitness (float): Current best individual function/fitness value
            params (Dict[str, Any]): Additional algorithm arguments

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray[float], Dict[str, Any]]:
                1. New population
                2. New population fitness/function values
                3. Additional arguments:
                    * loudness (numpy.ndarray[float]): Loudness.
                    * pulse_rates (numpy.ndarray[float]): Pulse rate.
                    * velocities (numpy.ndarray[float]): Velocities.

        """
        loudness = params.pop('loudness')
        pulse_rates = params.pop('pulse_rates')
        velocities = params.pop('velocities')

        for i in range(self.population_size):
            loudness[i], pulse_rates[i] = self.self_adaptation(loudness[i], pulse_rates[i])
            frequency = self.min_frequency + (self.max_frequency - self.min_frequency) * self.random()
            velocities[i] += (population[i] - best_x) * frequency
            if self.random() > pulse_rates[i]:
                solution = self.local_search(best=best_x, loudness=loudness[i], task=task, i=i, population=population)
            else:
                solution = task.repair(population[i] + velocities[i], rng=self.rng)
            new_fitness = task.eval(solution)
            if (new_fitness <= population_fitness[i]) and (self.random() < (self.min_loudness - loudness[i]) / self.starting_loudness):
                population[i], population_fitness[i] = solution, new_fitness
            if new_fitness <= best_fitness:
                best_x, best_fitness = solution.copy(), new_fitness
        return population, population_fitness, best_x, best_fitness, {'loudness': loudness, 'pulse_rates': pulse_rates, 'velocities': velocities}



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm
from niapy.util import full_array, euclidean

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')

__all__ = ['KrillHerd']


class KrillHerd(Algorithm):
    r"""Implementation of krill herd algorithm.

    Algorithm:
        Krill Herd Algorithm

    Date:
        2018

    Authors:
        Klemen Berkovič

    License:
        MIT

    Reference URL:
        http://www.sciencedirect.com/science/article/pii/S1007570412002171

    Reference paper:
        Amir Hossein Gandomi, Amir Hossein Alavi, Krill herd: A new bio-inspired optimization algorithm, Communications in Nonlinear Science and Numerical Simulation, Volume 17, Issue 12, 2012, Pages 4831-4845, ISSN 1007-5704, https://doi.org/10.1016/j.cnsns.2012.05.010.

    Attributes:
        Name (List[str]): List of strings representing algorithm names.
        population_size (int): Number of krill herds in population.
        N_max (float): Maximum induced speed.
        V_f (float): Foraging speed.
        D_max (float): Maximum diffusion speed.
        C_t (float): Constant :math:`\in [0, 2]`
        W_n (Union[int, float, numpy.ndarray]): Inertia weights of the motion induced from neighbors :math:`\in [0, 1]`.
        W_f (Union[int, float, numpy.ndarray]): Inertia weights of the motion induced from foraging :math`\in [0, 1]`.
        d_s (float): Maximum euclidean distance for neighbors.
        nn (int): Maximum neighbors for neighbors effect.
        epsilon (float): Small numbers for division.

    See Also:
        * :class:`niapy.algorithms.algorithm.Algorithm`

    """

    Name = ['KrillHerd', 'KH']

    @staticmethod
    def info():
        r"""Get basic information of algorithm.

        Returns:
            str: Basic information of algorithm.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""Amir Hossein Gandomi, Amir Hossein Alavi, Krill herd: A new bio-inspired optimization algorithm, Communications in Nonlinear Science and Numerical Simulation, Volume 17, Issue 12, 2012, Pages 4831-4845, ISSN 1007-5704, https://doi.org/10.1016/j.cnsns.2012.05.010."""

    def __init__(self, population_size=50, n_max=0.01, foraging_speed=0.02, diffusion_speed=0.002, c_t=0.93,
                 w_neighbor=0.42, w_foraging=0.38, d_s=2.63, max_neighbors=5, crossover_rate=0.2, mutation_rate=0.05,
                 *args, **kwargs):
        r"""Initialize KrillHerd.

        Args:
            population_size (Optional[int]): Number of krill herds in population.
            n_max (Optional[float]): Maximum induced speed.
            foraging_speed (Optional[float]): Foraging speed.
            diffusion_speed (Optional[float]): Maximum diffusion speed.
            c_t (Optional[float]): Constant $\in [0, 2]$.
            w_neighbor (Optional[Union[int, float, numpy.ndarray]]): Inertia weights of the motion induced from neighbors :math:`\in [0, 1]`.
            w_foraging (Optional[Union[int, float, numpy.ndarray]]): Inertia weights of the motion induced from foraging :math:`\in [0, 1]`.
            d_s (Optional[float]): Maximum euclidean distance for neighbors.
            max_neighbors (Optional[int]): Maximum neighbors for neighbors effect.
            crossover_rate (Optional[float]): Crossover probability.
            mutation_rate (Optional[float]): Mutation probability.

        See Also:
            * :func:`niapy.algorithms.algorithm.Algorithm.__init__`

        """
        super().__init__(population_size, *args, **kwargs)
        self.N_max = n_max
        self.V_f = foraging_speed
        self.D_max = diffusion_speed
        self.C_t = c_t
        self.W_n = w_neighbor
        self.W_f = w_foraging
        self.d_s = d_s
        self.nn = max_neighbors
        self._Cr = crossover_rate
        self._Mu = mutation_rate
        self.epsilon = np.finfo(float).eps

    def set_parameters(self, population_size=50, n_max=0.01, foraging_speed=0.02, diffusion_speed=0.002, c_t=0.93,
                       w_neighbor=0.42, w_foraging=0.38, d_s=2.63, max_neighbors=5, crossover_rate=0.2,
                       mutation_rate=0.05, **kwargs):
        r"""Set the arguments of an algorithm.

        Args:
            population_size (Optional[int]): Number of krill herds in population.
            n_max (Optional[float]): Maximum induced speed.
            foraging_speed (Optional[float]): Foraging speed.
            diffusion_speed (Optional[float]): Maximum diffusion speed.
            c_t (Optional[float]): Constant $\in [0, 2]$.
            w_neighbor (Optional[Union[int, float, numpy.ndarray]]): Inertia weights of the motion induced from neighbors :math:`\in [0, 1]`.
            w_foraging (Optional[Union[int, float, numpy.ndarray]]): Inertia weights of the motion induced from foraging :math:`\in [0, 1]`.
            d_s (Optional[float]): Maximum euclidean distance for neighbors.
            max_neighbors (Optional[int]): Maximum neighbors for neighbors effect.
            crossover_rate (Optional[float]): Crossover probability.
            mutation_rate (Optional[float]): Mutation probability.

        See Also:
            * :func:`niapy.algorithms.algorithm.Algorithm.set_parameters`

        """
        super().set_parameters(population_size=population_size, **kwargs)
        self.N_max = n_max
        self.V_f = foraging_speed
        self.D_max = diffusion_speed
        self.C_t = c_t
        self.W_n = w_neighbor
        self.W_f = w_foraging
        self.d_s = d_s
        self.nn = max_neighbors
        self._Cr = crossover_rate
        self._Mu = mutation_rate
        self.epsilon = np.finfo(float).eps

    def get_parameters(self):
        r"""Get parameter values for the algorithm.

        Returns:
            Dict[str, Any]: Algorithm parameters.

        """
        d = Algorithm.get_parameters(self)
        d.update({
            'N_max': self.N_max,
            'V_f': self.V_f,
            'D_max': self.D_max,
            'C_t': self.C_t,
            'W_n': self.W_n,
            'W_f': self.W_f,
            'd_s': self.d_s,
            'nn': self.nn,
        })
        return d

    def init_weights(self, task):
        r"""Initialize weights.

        Args:
            task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray]:
                1. Weights for neighborhood.
                2. Weights for foraging.

        """
        return full_array(self.W_n, task.dimension), full_array(self.W_f, task.dimension)

    def sense_range(self, ki, population):
        r"""Calculate sense range for selected individual.

        Args:
            ki (int): Selected individual.
            population (numpy.ndarray): Krill heard population.

        Returns:
            float: Sense range for krill.

        """
        return np.sum([euclidean(population[ki], population[i]) for i in range(self.population_size)]) / (self.nn * self.population_size)

    def get_neighbours(self, i, ids, population):
        r"""Get neighbours.

        Args:
            i (int): Individual looking for neighbours.
            ids (float): Maximal distance for being a neighbour.
            population (numpy.ndarray): Current population.

        Returns:
            numpy.ndarray: Neighbours of krill heard.

        """
        neighbors = list()
        for j in range(self.population_size):
            if j != i and ids > euclidean(population[i], population[j]):
                neighbors.append(j)
        if not neighbors:
            neighbors.append(self.integers(self.population_size))
        return np.asarray(neighbors)

    def get_x(self, x, y):
        r"""Get x values.

        Args:
            x (numpy.ndarray): First krill/individual.
            y (numpy.ndarray): Second krill/individual.

        Returns:
            numpy.ndarray: --

        """
        return ((y - x) + self.epsilon) / (euclidean(y, x) + self.epsilon)

    def get_k(self, x, y, b, w):
        r"""Get k values.

        Args:
            x (float): First krill/individual.
            y (float): Second krill/individual.
            b (float): Best krill/individual.
            w (float): Worst krill/individual.

        Returns:
            numpy.ndarray: K.

        """
        return ((x - y) + self.epsilon) / ((w - b) + self.epsilon)

    def induce_neighbors_motion(self, i, n, weights, population, population_fitness, best_index, worst_index, task):
        r"""Induced neighbours motion operator.

        Args:
            i (int): Index of individual being applied with operator.
            n:
            weights (numpy.ndarray[float]): Weights for this operator.
            population (numpy.ndarray): Current heard/population.
            population_fitness (numpy.ndarray[float]): Current populations/heard function/fitness values.
            best_index (numpy.ndarray): Current best krill in heard/population.
            worst_index (numpy.ndarray): Current worst krill in heard/population.
            task (Task): Optimization task.

        Returns:
            numpy.ndarray: Moved krill.

        """
        neighbor_i = self.get_neighbours(i, self.sense_range(i, population), population)
        neighbor_x, neighbor_f, f_b, f_w = population[neighbor_i], population_fitness[neighbor_i], population_fitness[best_index], population_fitness[worst_index]
        alpha_l = np.sum(
            np.asarray([self.get_k(population_fitness[i], j, f_b, f_w) for j in neighbor_f]) * np.asarray([self.get_x(population[i], j) for j in neighbor_x]).T)
        alpha_t = 2 * (1 + self.random() * (task.iters + 1) / task.max_iters)
        return self.N_max * (alpha_l + alpha_t) + weights * n

    def induce_foraging_motion(self, i, x, x_f, f, weights, population, population_fitness, best_index, worst_index, task):
        r"""Induced foraging motion operator.

        Args:
            i (int): Index of current krill being operated.
            x (numpy.ndarray): Position of food.
            x_f (float): Fitness/function values of food.
            f:
            weights (numpy.ndarray[float]): Weights for this operator.
            population (numpy.ndarray):  Current population/heard.
            population_fitness (numpy.ndarray[float]): Current heard/populations function/fitness values.
            best_index (numpy.ndarray): Index of current best krill in heard.
            worst_index (numpy.ndarray): Index of current worst krill in heard.
            task (Task): Optimization task.

        Returns:
            numpy.ndarray: Moved krill.

        """
        beta_f = 2 * (1 - (task.iters + 1) / task.max_iters) * self.get_k(population_fitness[i], x_f, population_fitness[best_index], population_fitness[worst_index]) * self.get_x(
            population[i], x) if population_fitness[best_index] < population_fitness[i] else 0
        beta_b = self.get_k(population_fitness[i], population_fitness[best_index], population_fitness[best_index], population_fitness[worst_index]) * self.get_x(population[i], population[best_index])
        return self.V_f * (beta_f + beta_b) + weights * f

    def induce_physical_diffusion(self, task):
        r"""Induced physical diffusion operator.

        Args:
            task (Task): Optimization task.

        Returns:
            numpy.ndarray:

        """
        return self.D_max * (1 - (task.iters + 1) / task.max_iters) * self.uniform(-1, 1, task.dimension)

    def delta_t(self, task):
        r"""Get new delta for all dimensions.

        Args:
            task (Task): Optimization task.

        Returns:
            numpy.ndarray: --

        """
        return self.C_t * np.sum(task.range)

    def crossover(self, x, xo, crossover_rate):
        r"""Crossover operator.

        Args:
            x (numpy.ndarray): Krill/individual being applied with operator.
            xo (numpy.ndarray): Krill/individual being used in conjunction within operator.
            crossover_rate (float): Crossover probability.

        Returns:
            numpy.ndarray: New krill/individual.

        """
        return [xo[i] if self.random() < crossover_rate else x[i] for i in range(len(x))]

    def mutate(self, x, x_b, mutation_rate):
        r"""Mutate operator.

        Args:
            x (numpy.ndarray): Individual being mutated.
            x_b (numpy.ndarray): Global best individual.
            mutation_rate (float): Probability of mutations.

        Returns:
            numpy.ndarray: Mutated krill.

        """
        return [x[i] if self.random() < mutation_rate else (x_b[i] + self.random()) for i in range(len(x))]

    def get_food_location(self, population, population_fitness, task):
        r"""Get food location for krill heard.

        Args:
            population (numpy.ndarray): Current heard/population.
            population_fitness (numpy.ndarray[float]): Current heard/populations function/fitness values.
            task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray, float]:
                1. Location of food.
                2. Foods function/fitness value.

        """
        x_food = task.repair(np.asarray([np.sum(population[:, i] / population_fitness) for i in range(task.dimension)]) / np.sum(1 / population_fitness),
                             rng=self.rng)
        x_food_f = task.eval(x_food)
        return x_food, x_food_f

    def mutation_rate(self, xf, yf, xf_best, xf_worst):
        r"""Get mutation probability.

        Args:
            xf (float):
            yf (float):
            xf_best (float):
            xf_worst (float):

        Returns:
            float: New mutation probability.

        """
        return self._Mu / (self.get_k(xf, yf, xf_best, xf_worst) + 1e-31)

    def crossover_rate(self, xf, yf, xf_best, xf_worst):
        r"""Get crossover probability.

        Args:
            xf (float):
            yf (float):
            xf_best (float):
            xf_worst (float):

        Returns:
            float: New crossover probability.

        """
        return self._Cr * self.get_k(xf, yf, xf_best, xf_worst)

    def init_population(self, task):
        r"""Initialize stating population.

        Args:
            task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, Dict[str, Any]]:
                1. Initialized population.
                2. Initialized populations function/fitness values.
                3. Additional arguments:
                    * w_neighbor (numpy.ndarray): Weights neighborhood.
                    * w_foraging (numpy.ndarray): Weights foraging.
                    * induced_speed (numpy.ndarray): Induced speed.
                    * foraging_speed (numpy.ndarray): Foraging speed.

        See Also:
            * :func:`niapy.algorithms.algorithm.Algorithm.init_population`

        """
        krill_herd, krill_herd_fitness, d = Algorithm.init_population(self, task)
        w_neighbor, w_foraging = self.init_weights(task)
        induced_speed, foraging_speed = np.zeros(self.population_size), np.zeros(self.population_size)
        d.update({'w_neighbor': w_neighbor, 'w_foraging': w_foraging, 'induced_speed': induced_speed, 'foraging_speed': foraging_speed})
        return krill_herd, krill_herd_fitness, d

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of KrillHerd algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current heard/population.
            population_fitness (numpy.ndarray[float]): Current heard/populations function/fitness values.
            best_x (numpy.ndarray): Global best individual.
            best_fitness (float): Global best individuals function fitness values.
            **params (Dict[str, Any]): Additional arguments.

        Returns:
            Tuple [numpy.ndarray, numpy.ndarray, numpy.ndarray, float Dict[str, Any]]:
                1. New herd/population
                2. New herd/populations function/fitness values.
                3. New global best solution.
                4. New global best solutions fitness/objective value.
                5. Additional arguments:
                    * w_neighbor (numpy.ndarray): --
                    * w_foraging (numpy.ndarray): --
                    * induced_speed (numpy.ndarray): --
                    * foraging_speed (numpy.ndarray): --

        """
        w_neighbor = params.pop('w_neighbor')
        w_foraging = params.pop('w_foraging')
        induced_speed = params.pop('induced_speed')
        foraging_speed = params.pop('foraging_speed')

        ikh_b, ikh_w = np.argmin(population_fitness), np.argmax(population_fitness)
        x_food, x_food_f = self.get_food_location(population, population_fitness, task)
        if x_food_f < best_fitness:
            best_x, best_fitness = x_food, x_food_f  # noqa: F841
        induced_speed = np.asarray([self.induce_neighbors_motion(i, induced_speed[i], w_neighbor, population, population_fitness, ikh_b, ikh_w, task) for i in range(self.population_size)])
        foraging_speed = np.asarray([self.induce_foraging_motion(i, x_food, x_food_f, foraging_speed[i], w_foraging, population, population_fitness, ikh_b, ikh_w, task) for i in range(self.population_size)])
        diffusion = np.asarray([self.induce_physical_diffusion(task) for _ in range(self.population_size)])
        new_herd = population + (self.delta_t(task) * (induced_speed + foraging_speed + diffusion))
        crossover_rates = np.asarray([self.crossover_rate(population_fitness[i], population_fitness[ikh_b], population_fitness[ikh_b], population_fitness[ikh_w]) for i in range(self.population_size)])
        new_herd = np.asarray([self.crossover(new_herd[i], population[i], crossover_rates[i]) for i in range(self.population_size)])
        mutation_rates = np.asarray([self.mutation_rate(population_fitness[i], population_fitness[ikh_b], population_fitness[ikh_b], population_fitness[ikh_w]) for i in range(self.population_size)])
        new_herd = np.asarray([self.mutate(new_herd[i], population[ikh_b], mutation_rates[i]) for i in range(self.population_size)])
        population = np.apply_along_axis(task.repair, 1, new_herd, rng=self.rng)
        population_fitness = np.apply_along_axis(task.eval, 1, population)
        best_x, best_fitness = self.get_best(population, population_fitness, best_x, best_fitness)
        return population, population_fitness, best_x, best_fitness, {'w_neighbor': w_neighbor, 'w_foraging': w_foraging, 'induced_speed': induced_speed, 'foraging_speed': foraging_speed}



# encoding=utf8
import logging

from niapy.algorithms.algorithm import Algorithm

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.other')
logger.setLevel('INFO')

__all__ = ['HillClimbAlgorithm']


def neighborhood(x, delta, task, rng):
    r"""Get neighbours of point.

    Args:
        x (numpy.ndarray): Point.
        delta (float): Standard deviation.
        task (Task): Optimization task.
        rng (numpy.random.Generator): Random generator.

    Returns:
        Tuple[numpy.ndarray, float]:
            1. New solution.
            2. New solutions function/fitness value.

    """
    new_x = x + rng.normal(0, delta, task.dimension)
    new_x = task.repair(new_x, rng)
    new_x_fitness = task.eval(new_x)
    return new_x, new_x_fitness


class HillClimbAlgorithm(Algorithm):
    r"""Implementation of iterative hill climbing algorithm.

    Algorithm:
        Hill Climbing Algorithm

    Date:
        2018

    Authors:
        Jan Popič

    License:
        MIT

    Reference URL:

    Reference paper:

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    Attributes:
        delta (float): Change for searching in neighborhood.
        neighborhood_function (Callable[numpy.ndarray, float, Task], Tuple[numpy.ndarray, float]]): Function for getting neighbours.

    """

    Name = ['HillClimbAlgorithm', 'HC']

    @staticmethod
    def info():
        r"""Get basic information about the algorithm.

        Returns:
            str: Basic information.

        See Also:
            :func:`niapy.algorithms.algorithm.Algorithm.info`

        """
        return r"""TODO"""

    def __init__(self, delta=0.5, neighborhood_function=neighborhood, *args, **kwargs):
        """Initialize HillClimbAlgorithm.

        Args:
            * delta (Optional[float]): Change for searching in neighborhood.
            * neighborhood_function (Optional[Callable[numpy.ndarray, float, Task], Tuple[numpy.ndarray, float]]]): Function for getting neighbours.

        """
        kwargs.pop('population_size', None)
        super().__init__(1, *args, **kwargs)
        self.delta = delta
        self.neighborhood_function = neighborhood_function

    def set_parameters(self, delta=0.5, neighborhood_function=neighborhood, **kwargs):
        r"""Set the algorithm parameters/arguments.

        Args:
            * delta (Optional[float]): Change for searching in neighborhood.
            * neighborhood_function (Optional[Callable[numpy.ndarray, float, Task], Tuple[numpy.ndarray, float]]]): Function for getting neighbours.

        """
        kwargs.pop('population_size', None)
        super().set_parameters(population_size=1, **kwargs)
        self.delta = delta
        self.neighborhood_function = neighborhood_function

    def get_parameters(self):
        d = Algorithm.get_parameters(self)
        d.update({
            'delta': self.delta,
            'neighborhood_function': self.neighborhood_function
        })
        return d

    def init_population(self, task):
        r"""Initialize stating point.

        Args:
            task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray, float, Dict[str, Any]]:
                1. New individual.
                2. New individual function/fitness value.
                3. Additional arguments.

        """
        x = task.lower + self.random(task.dimension) * task.range
        return x, task.eval(x), {}

    def run_iteration(self, task, x, fx, best_x, best_fitness, **params):
        r"""Core function of HillClimbAlgorithm algorithm.

        Args:
            task (Task): Optimization task.
            x (numpy.ndarray): Current solution.
            fx (float): Current solutions fitness/function value.
            best_x (numpy.ndarray): Global best solution.
            best_fitness (float): Global best solutions function/fitness value.
            **params (Dict[str, Any]): Additional arguments.

        Returns:
            Tuple[numpy.ndarray, float, numpy.ndarray, float, Dict[str, Any]]:
                1. New solution.
                2. New solutions function/fitness value.
                3. Additional arguments.

        """
        lo, xn = False, task.lower + task.range * self.random(task.dimension)
        xn_f = task.eval(xn)
        while not lo:
            yn, yn_f = self.neighborhood_function(x, self.delta, task, rng=self.rng)
            if yn_f < xn_f:
                xn, xn_f = yn, yn_f
            else:
                lo = True or task.stopping_condition()
        best_x, best_fitness = self.get_best(xn, xn_f, best_x, best_fitness)
        return xn, xn_f, best_x, best_fitness, {}



# encoding=utf8

"""Module with implementations of basic and hybrid algorithms."""

from niapy.algorithms import basic
from niapy.algorithms import modified
from niapy.algorithms import other
from niapy.algorithms.algorithm import Algorithm, Individual, default_numpy_init, default_individual_init

__all__ = [
    'basic',
    'modified',
    'other',
    'Algorithm',
    'default_numpy_init',
    'default_individual_init',
    'Individual',
]



#!/usr/bin/env python

"""Setup script for the package."""

import io
import os
import sys
import logging

import setuptools


PACKAGE_NAME = 'niapy'
MINIMUM_PYTHON_VERSION = (3, 6)


def check_python_version():
    """Exit when the Python version is too low."""
    if sys.version_info[:2] < MINIMUM_PYTHON_VERSION:
        sys.exit("Python {}.{}+ is required.".format(*MINIMUM_PYTHON_VERSION))


def read_package_variable(key, filename='__init__.py'):
    """Read the value of a variable from the package without importing."""
    module_path = os.path.join(PACKAGE_NAME, filename)
    with open(module_path) as module:
        for line in module:
            parts = line.strip().split(' ', 2)
            if parts[:-1] == [key, '=']:
                return parts[-1].strip("'")
    logging.warning("'%s' not found in '%s'", key, module_path)
    return None


def build_description():
    """Build a description for the project from documentation files."""
    try:
        readme = io.open("README.rst", encoding="UTF-8").read()
    except IOError:
        return "<placeholder>"
    else:
        return readme


check_python_version()

PACKAGE_VERSION = read_package_variable('__version__')

setuptools.setup(
    name=PACKAGE_NAME,
    version="2.0.1",
    description="Python micro framework for building nature-inspired algorithms.",
    url='https://github.com/NiaOrg/NiaPy',
    author='NiaOrg',
    author_email='niapy.organization@gmail.com',
    packages=setuptools.find_packages(exclude=['*tests*']),
    long_description=build_description(),
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development'
    ],
    tests_require=[
        'flake8 ~= 3.7.7',
        'astroid >= 2.0.4',
        'pytest ~= 3.7.1',
        'coverage ~= 4.4.2',
        'coverage-space ~= 1.0.2',
        'bandit < 1.7.3'
    ],
    install_requires=[
        'numpy >= 1.17.0',
        'matplotlib >= 2.2.4',
        'pandas >= 0.24.2',
        'openpyxl >= 3.0.3',
    ]
)



# encoding=utf8
import logging

import numpy as np

from niapy.algorithms.algorithm import Algorithm

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')

__all__ = ['MothFlameOptimizer']


class MothFlameOptimizer(Algorithm):
    r"""MothFlameOptimizer of Moth flame optimizer.

    Algorithm:
        Moth flame optimizer

    Date:
        2018

    Author:
        Kivanc Guckiran and Klemen Berkovič

    License:
        MIT

    Reference paper:
        Mirjalili, Seyedali. "Moth-flame optimization algorithm: A novel nature-inspired heuristic paradigm." Knowledge-Based Systems 89 (2015): 228-249.

    Attributes:
        Name (List[str]): List of strings representing algorithm name.

    See Also:
        * :class:`niapy.algorithms.algorithm.Algorithm`

    """

    Name = ['MothFlameOptimizer', 'MFO']

    @staticmethod
    def info():
        r"""Get basic information of algorithm.

        Returns:
            str: Basic information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""Mirjalili, Seyedali. "Moth-flame optimization algorithm: A novel nature-inspired heuristic paradigm." Knowledge-Based Systems 89 (2015): 228-249."""

    def run_iteration(self, task, population, population_fitness, best_x, best_fitness, **params):
        r"""Core function of MothFlameOptimizer algorithm.

        Args:
            task (Task): Optimization task.
            population (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray): Current population fitness/function values.
            best_x (numpy.ndarray): Current population best individual.
            best_fitness (float): Current best individual.
            **params (Dict[str, Any]): Additional parameters

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population.
                2. New population fitness/function values.
                3. New global best solution.
                4. New global best fitness/objective value.
                5. Additional arguments:
                    * best_flames (numpy.ndarray): Best individuals.
                    * best_flame_fitness (numpy.ndarray): Best individuals fitness/function values.
                    * previous_population (numpy.ndarray): Previous population.
                    * previous_fitness (numpy.ndarray): Previous population fitness/function values.

        """
        # Previous positions
        # Create sorted population
        indexes = np.argsort(population_fitness)
        sorted_population = population[indexes]
        # Some parameters
        flame_no, a = round(self.population_size - (task.iters + 1) * ((self.population_size - 1) / task.max_iters)), -1 + (task.iters + 1) * (
                    (-1) / task.max_iters)
        for i in range(self.population_size):
            for j in range(task.dimension):
                distance_to_flame, b, t = abs(sorted_population[i, j] - population[i, j]), 1, (a - 1) * self.random() + 1
                if i <= flame_no:
                    population[i, j] = distance_to_flame * np.exp(b * t) * np.cos(2 * np.pi * t) + sorted_population[i, j]
                else:
                    population[i, j] = distance_to_flame * np.exp(b * t) * np.cos(2 * np.pi * t) + sorted_population[
                        flame_no, j]
        population = np.apply_along_axis(task.repair, 1, population, self.rng)
        population_fitness = np.apply_along_axis(task.eval, 1, population)
        best_x, best_fitness = self.get_best(population, population_fitness, best_x, best_fitness)
        return population, population_fitness, best_x, best_fitness, {}



"""Implementation of basic nature-inspired algorithms."""

from niapy.algorithms.basic.abc import ArtificialBeeColonyAlgorithm
from niapy.algorithms.basic.ba import BatAlgorithm
from niapy.algorithms.basic.bea import BeesAlgorithm
from niapy.algorithms.basic.bfo import BacterialForagingOptimization
from niapy.algorithms.basic.ca import CamelAlgorithm
from niapy.algorithms.basic.cro import CoralReefsOptimization
from niapy.algorithms.basic.cs import CuckooSearch
from niapy.algorithms.basic.cso import CatSwarmOptimization
from niapy.algorithms.basic.de import DifferentialEvolution, MultiStrategyDifferentialEvolution, \
    DynNpDifferentialEvolution, AgingNpDifferentialEvolution, DynNpMultiStrategyDifferentialEvolution, multi_mutations
from niapy.algorithms.basic.es import EvolutionStrategy1p1, EvolutionStrategyMp1, EvolutionStrategyMpL, \
    EvolutionStrategyML
from niapy.algorithms.basic.fa import FireflyAlgorithm
from niapy.algorithms.basic.foa import ForestOptimizationAlgorithm
from niapy.algorithms.basic.fpa import FlowerPollinationAlgorithm
from niapy.algorithms.basic.fss import FishSchoolSearch
from niapy.algorithms.basic.fwa import FireworksAlgorithm, EnhancedFireworksAlgorithm, DynamicFireworksAlgorithm, \
    DynamicFireworksAlgorithmGauss, BareBonesFireworksAlgorithm
from niapy.algorithms.basic.ga import GeneticAlgorithm
from niapy.algorithms.basic.gsa import GravitationalSearchAlgorithm
from niapy.algorithms.basic.gso import GlowwormSwarmOptimization, GlowwormSwarmOptimizationV1, \
    GlowwormSwarmOptimizationV2, GlowwormSwarmOptimizationV3
from niapy.algorithms.basic.gwo import GreyWolfOptimizer
from niapy.algorithms.basic.hho import HarrisHawksOptimization
from niapy.algorithms.basic.hs import HarmonySearch, HarmonySearchV1
from niapy.algorithms.basic.kh import KrillHerd
from niapy.algorithms.basic.loa import LionOptimizationAlgorithm
from niapy.algorithms.basic.mbo import MonarchButterflyOptimization
from niapy.algorithms.basic.mfo import MothFlameOptimizer
from niapy.algorithms.basic.mke import MonkeyKingEvolutionV1, MonkeyKingEvolutionV2, MonkeyKingEvolutionV3
from niapy.algorithms.basic.pso import ParticleSwarmAlgorithm, ParticleSwarmOptimization, \
    CenterParticleSwarmOptimization, ComprehensiveLearningParticleSwarmOptimizer, \
    OppositionVelocityClampingParticleSwarmOptimization, MutatedCenterParticleSwarmOptimization, \
    MutatedCenterUnifiedParticleSwarmOptimization, MutatedParticleSwarmOptimization
from niapy.algorithms.basic.sca import SineCosineAlgorithm
from niapy.algorithms.basic.clonalg import ClonalSelectionAlgorithm

__all__ = [
    'BatAlgorithm',
    'FireflyAlgorithm',
    'DifferentialEvolution',
    'AgingNpDifferentialEvolution',
    'DynNpDifferentialEvolution',
    'MultiStrategyDifferentialEvolution',
    'DynNpMultiStrategyDifferentialEvolution',
    'multi_mutations',
    'FlowerPollinationAlgorithm',
    'GreyWolfOptimizer',
    'CatSwarmOptimization',
    'GeneticAlgorithm',
    'ArtificialBeeColonyAlgorithm',
    'ParticleSwarmAlgorithm',
    'BareBonesFireworksAlgorithm',
    'CamelAlgorithm',
    'MonkeyKingEvolutionV1',
    'MonkeyKingEvolutionV2',
    'MonkeyKingEvolutionV3',
    'EvolutionStrategy1p1',
    'EvolutionStrategyMp1',
    'EvolutionStrategyMpL',
    'EvolutionStrategyML',
    'SineCosineAlgorithm',
    'GlowwormSwarmOptimization',
    'GlowwormSwarmOptimizationV1',
    'GlowwormSwarmOptimizationV2',
    'GlowwormSwarmOptimizationV3',
    'HarmonySearch',
    'HarmonySearchV1',
    'KrillHerd',
    'LionOptimizationAlgorithm',
    'FireworksAlgorithm',
    'EnhancedFireworksAlgorithm',
    'DynamicFireworksAlgorithm',
    'DynamicFireworksAlgorithmGauss',
    'GravitationalSearchAlgorithm',
    'MothFlameOptimizer',
    'FishSchoolSearch',
    'CuckooSearch',
    'CoralReefsOptimization',
    'ForestOptimizationAlgorithm',
    'MonarchButterflyOptimization',
    'BeesAlgorithm',
    'ParticleSwarmOptimization',
    'MutatedParticleSwarmOptimization',
    'MutatedCenterUnifiedParticleSwarmOptimization',
    'MutatedCenterParticleSwarmOptimization',
    'OppositionVelocityClampingParticleSwarmOptimization',
    'ComprehensiveLearningParticleSwarmOptimizer',
    'CenterParticleSwarmOptimization',
    'HarrisHawksOptimization',
    'BacterialForagingOptimization',
    'ClonalSelectionAlgorithm'
]



# encoding=utf8

"""Implementations of Trid function."""

import numpy as np
from niapy.problems.problem import Problem

__all__ = ['Trid']


class Trid(Problem):
    r"""Implementations of Trid functions.

    Date: 2018

    Author: Klemen Berkovič

    License: MIT

    Function:
    **Trid Function**

        :math:`f(\textbf{x}) = \sum_{i = 1}^D \left( x_i - 1 \right)^2 - \sum_{i = 2}^D x_i x_{i - 1}`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-D^2, D^2]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:**
        :math:`f(\textbf{x}^*) = \frac{-D(D + 4)(D - 1)}{6}` at :math:`\textbf{x}^* = (1 (D + 1 - 1), \cdots , i (D + 1 - i) , \cdots , D (D + 1 - D))`

    LaTeX formats:
        Inline:
                $f(\textbf{x}) = \sum_{i = 1}^D \left( x_i - 1 \right)^2 - \sum_{i = 2}^D x_i x_{i - 1}$

        Equation:
                \begin{equation} f(\textbf{x}) = \sum_{i = 1}^D \left( x_i - 1 \right)^2 - \sum_{i = 2}^D x_i x_{i - 1} \end{equation}

        Domain:
                $-D^2 \leq x_i \leq D^2$

    Reference:
        https://www.sfu.ca/~ssurjano/trid.html

    """

    def __init__(self, dimension=4, *args, **kwargs):
        r"""Initialize Trid problem..

        Args:
            dimension (Optional[int]): Dimension of the problem.

        See Also:
            :func:`niapy.problems.Problem.__init__`

        """
        kwargs.pop('lower', None)
        kwargs.pop('upper', None)
        super().__init__(dimension, -(dimension ** 2), dimension ** 2, *args, **kwargs)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = \sum_{i = 1}^D \left( x_i - 1 \right)^2 - \sum_{i = 2}^D x_i x_{i - 1}$'''

    def _evaluate(self, x):
        sum1 = np.sum((x - 1) ** 2)
        sum2 = np.sum(x[1:] * x[:-1])
        return sum1 - sum2



# encoding=utf8
import logging
from math import ceil

import numpy as np

from niapy.algorithms.algorithm import Algorithm, Individual, default_individual_init
from niapy.util import objects_to_array

logging.basicConfig()
logger = logging.getLogger('niapy.algorithms.basic')
logger.setLevel('INFO')

__all__ = ['EvolutionStrategy1p1', 'EvolutionStrategyMp1', 'EvolutionStrategyMpL', 'EvolutionStrategyML']


class IndividualES(Individual):
    r"""Individual for Evolution Strategies.

    See Also:
        * :class:`niapy.algorithms.Individual`

    """

    def __init__(self, rho=1, **kwargs):
        r"""Initialize individual.

        Args:
            rho(Optional[int]): Rho parameter.

        See Also:
            * :func:`niapy.algorithms.Individual.__init__`

        """
        super().__init__(**kwargs)
        self.rho = rho


class EvolutionStrategy1p1(Algorithm):
    r"""Implementation of (1 + 1) evolution strategy algorithm. Uses just one individual.

    Algorithm:
        (1 + 1) Evolution Strategy Algorithm

    Date:
        2018

    Authors:
        Klemen Berkovič

    License:
        MIT

    Reference URL:

    Reference paper:
        KALYANMOY, Deb. "Multi-Objective optimization using evolutionary algorithms".
        John Wiley & Sons, Ltd. Kanpur, India. 2001.

    Attributes:
        Name (List[str]): List of strings representing algorithm names.
        mu (int): Number of parents.
        k (int): Number of iterations before checking and fixing rho.
        c_a (float): Search range amplification factor.
        c_r (float): Search range reduction factor.

    See Also:
        * :class:`niapy.algorithms.Algorithm`

    """

    Name = ['EvolutionStrategy1p1', 'EvolutionStrategy(1+1)', 'ES(1+1)']

    @staticmethod
    def info():
        r"""Get algorithms information.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""KALYANMOY, Deb. "Multi-Objective optimization using evolutionary algorithms". John Wiley & Sons, Ltd. Kanpur, India. 2001."""

    def __init__(self, mu=1, k=10, c_a=1.1, c_r=0.5, epsilon=1e-20, *args, **kwargs):
        """Initialize EvolutionStrategy1p1.

        Args:
            mu (Optional[int]): Number of parents
            k (Optional[int]): Number of iterations before checking and fixing rho
            c_a (Optional[float]): Search range amplification factor
            c_r (Optional[float]): Search range reduction factor
            epsilon (Optional[float]): Small number.

        See Also:
            * :func:`niapy.algorithms.Algorithm.__init__`

        """
        kwargs.pop('population_size', None)
        super().__init__(population_size=mu, individual_type=kwargs.pop('individual_type', IndividualES), *args, **kwargs)
        self.mu = mu
        self.k = k
        self.c_a = c_a
        self.c_r = c_r
        self.epsilon = epsilon

    def set_parameters(self, mu=1, k=10, c_a=1.1, c_r=0.5, epsilon=1e-20, **kwargs):
        r"""Set the arguments of an algorithm.

        Arguments:
            mu (Optional[int]): Number of parents
            k (Optional[int]): Number of iterations before checking and fixing rho
            c_a (Optional[float]): Search range amplification factor
            c_r (Optional[float]): Search range reduction factor
            epsilon (Optional[float]): Small number.

        See Also:
            * :func:`niapy.algorithms.Algorithm.set_parameters`

        """
        kwargs.pop('population_size', None)
        super().set_parameters(population_size=mu, individual_type=kwargs.pop('individual_type', IndividualES), **kwargs)
        self.mu = mu
        self.k = k
        self.c_a = c_a
        self.c_r = c_r
        self.epsilon = epsilon

    def mutate(self, x, rho):
        r"""Mutate individual.

        Args:
            x (numpy.ndarray): Current individual.
            rho (float): Current standard deviation.

        Returns:
            Individual: Mutated individual.

        """
        return x + self.normal(0, rho, len(x))

    def update_rho(self, rho, k):
        r"""Update standard deviation.

        Args:
            rho (float): Current standard deviation.
            k (int): Number of successful mutations.

        Returns:
            float: New standard deviation.

        """
        phi = k / self.k
        if phi < 0.2:
            return self.c_r * rho if rho > self.epsilon else 1
        elif phi > 0.2:
            return self.c_a * rho if rho > self.epsilon else 1
        else:
            return rho

    def init_population(self, task):
        r"""Initialize starting individual.

        Args:
            task (Task): Optimization task.

        Returns:
            Tuple[Individual, float, Dict[str, Any]]:
                1. Initialized individual.
                2. Initialized individual fitness/function value.
                3. Additional arguments:
                    * ki (int): Number of successful rho update.

        """
        c, ki = IndividualES(task=task, rng=self.rng), 0
        return c, c.f, {'ki': ki}

    def run_iteration(self, task, c, population_fitness, best_x, best_fitness, **params):
        r"""Core function of EvolutionStrategy(1+1) algorithm.

        Args:
            task (Task): Optimization task.
            c (Individual): Current position.
            population_fitness (float): Current position function/fitness value.
            best_x (numpy.ndarray): Global best position.
            best_fitness (float): Global best function/fitness value.
            **params (Dict[str, Any]): Additional arguments.

        Returns:
            Tuple[Individual, float, Individual, float, Dict[str, Any]]:
                1. Initialized individual.
                2. Initialized individual fitness/function value.
                3. New global best solution.
                4. New global best solutions fitness/objective value.
                5. Additional arguments:
                    * ki (int): Number of successful rho update.

        """
        ki = params.pop('ki')

        if (task.iters + 1) % self.k == 0:
            c.rho, ki = self.update_rho(c.rho, ki), 0
        cn = objects_to_array([task.repair(self.mutate(c.x, c.rho), self.rng) for _i in range(self.mu)])
        cn_f = np.asarray([task.eval(cn[i]) for i in range(len(cn))])
        ib = np.argmin(cn_f)
        if cn_f[ib] < c.f:
            c.x, c.f, ki = cn[ib], cn_f[ib], ki + 1
            if cn_f[ib] < best_fitness:
                best_x, best_fitness = self.get_best(cn[ib], cn_f[ib], best_x, best_fitness)
        return c, c.f, best_x, best_fitness, {'ki': ki}


class EvolutionStrategyMp1(EvolutionStrategy1p1):
    r"""Implementation of (mu + 1) evolution strategy algorithm. Algorithm creates mu mutants but into new generation goes only one individual.

    Algorithm:
        (:math:`\mu + 1`) Evolution Strategy Algorithm

    Date:
        2018

    Authors:
        Klemen Berkovič

    License:
        MIT

    Reference URL:

    Reference paper:

    Attributes:
        Name (List[str]): List of strings representing algorithm names.

    See Also:
        * :class:`niapy.algorithms.basic.EvolutionStrategy1p1`

    """

    Name = ['EvolutionStrategyMp1', 'EvolutionStrategy(mu+1)', 'ES(m+1)']

    @staticmethod
    def info():
        r"""Get algorithms information.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""TODO"""

    def __init__(self, mu=40, *args, **kwargs):
        """Initialize EvolutionStrategyMp1."""
        super().__init__(mu=mu, *args, **kwargs)

    def set_parameters(self, **kwargs):
        r"""Set core parameters of EvolutionStrategy(mu+1) algorithm.

        See Also:
            * :func:`niapy.algorithms.basic.EvolutionStrategy1p1.set_parameters`

        """
        mu = kwargs.pop('mu', 40)
        super().set_parameters(self, mu=mu, **kwargs)


class EvolutionStrategyMpL(EvolutionStrategy1p1):
    r"""Implementation of (mu + lambda) evolution strategy algorithm. Mutation creates lambda individual. Lambda individual compete with mu individuals for survival, so only mu individual go to new generation.

    Algorithm:
        (:math:`\mu + \lambda`) Evolution Strategy Algorithm

    Date:
        2018

    Authors:
        Klemen Berkovič

    License:
        MIT

    Reference URL:

    Reference paper:

    Attributes:
        Name (List[str]): List of strings representing algorithm names
        lam (int): Lambda.

    See Also:
        * :class:`niapy.algorithms.basic.EvolutionStrategy1p1`

    """

    Name = ['EvolutionStrategyMpL', 'EvolutionStrategy(mu+lambda)', 'ES(m+l)']

    @staticmethod
    def info():
        r"""Get algorithms information.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""TODO"""

    def __init__(self, lam=45, *args, **kwargs):
        """Initialize EvolutionStrategyMpL.

        Args:
            lam (int): Number of new individual generated by mutation.

        """
        super().__init__(initialization_function=default_individual_init, *args, **kwargs)
        self.lam = lam

    def set_parameters(self, lam=45, **kwargs):
        r"""Set the arguments of an algorithm.

        Args:
            lam (int): Number of new individual generated by mutation.

        See Also:
            * :func:`niapy.algorithms.basic.es.EvolutionStrategy1p1.set_parameters`

        """
        super().set_parameters(initialization_function=default_individual_init, **kwargs)
        self.lam = lam

    def update_rho(self, pop, k):
        r"""Update standard deviation for population.

        Args:
            pop (numpy.ndarray[Individual]): Current population.
            k (int): Number of successful mutations.

        """
        phi = k / self.k
        if phi < 0.2:
            for i in pop:
                i.rho = self.c_r * i.rho
        elif phi > 0.2:
            for i in pop:
                i.rho = self.c_a * i.rho

    @staticmethod
    def change_count(c, cn):
        r"""Update number of successful mutations for population.

        Args:
            c (numpy.ndarray[Individual]): Current population.
            cn (numpy.ndarray[Individual]): New population.

        Returns:
            int: Number of successful mutations.

        """
        k = 0
        for e in cn:
            if e not in c.tolist():
                k += 1
        return k

    def mutate_rand(self, pop, task):
        r"""Mutate random individual form population.

        Args:
            pop (numpy.ndarray[Individual]): Current population.
            task (Task): Optimization task.

        Returns:
            numpy.ndarray: Random individual from population that was mutated.

        """
        i = self.integers(self.mu)
        return task.repair(self.mutate(pop[i].x, pop[i].rho), rng=self.rng)

    def init_population(self, task):
        r"""Initialize starting population.

        Args:
            task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray[Individual], numpy.ndarray[float], Dict[str, Any]]:
                1. Initialized population.
                2. Initialized populations function/fitness values.
                3. Additional arguments:
                    * ki (int): Number of successful mutations.

        See Also:
            * :func:`niapy.algorithms.algorithm.Algorithm.init_population`

        """
        c, fc, d = Algorithm.init_population(self, task)
        d.update({'ki': 0})
        return c, fc, d

    def run_iteration(self, task, c, population_fitness, best_x, best_fitness, **params):
        r"""Core function of EvolutionStrategyMpL algorithm.

        Args:
            task (Task): Optimization task.
            c (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray): Current populations fitness/function values.
            best_x (numpy.ndarray): Global best individual.
            best_fitness (float): Global best individuals fitness/function value.
            **params (Dict[str, Any]): Additional arguments.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population.
                2. New populations function/fitness values.
                3. New global best solution.
                4. New global best solutions fitness/objective value.
                5. Additional arguments:
                    * ki (int): Number of successful mutations.

        """
        ki = params.pop('ki')

        if (task.iters + 1) % self.k == 0:
            _, ki = self.update_rho(c, ki), 0
        cn = objects_to_array(
            [IndividualES(x=self.mutate_rand(c, task), task=task, rng=self.rng) for _ in range(self.lam)])
        cn = np.append(cn, c)
        cn = objects_to_array([cn[i] for i in np.argsort([i.f for i in cn])[:self.mu]])
        ki += self.change_count(c, cn)
        fcn = np.asarray([x.f for x in cn])
        best_x, best_fitness = self.get_best(cn, fcn, best_x, best_fitness)
        return cn, fcn, best_x, best_fitness, {'ki': ki}


class EvolutionStrategyML(EvolutionStrategyMpL):
    r"""Implementation of (mu, lambda) evolution strategy algorithm. Algorithm is good for dynamic environments. Mu individual create lambda children. Only best mu children go to new generation. Mu parents are discarded.

    Algorithm:
        (:math:`\mu + \lambda`) Evolution Strategy Algorithm

    Date:
        2018

    Authors:
        Klemen Berkovič

    License:
        MIT

    Reference URL:

    Reference paper:

    Attributes:
        Name (List[str]): List of strings representing algorithm names

    See Also:
        * :class:`niapy.algorithm.basic.es.EvolutionStrategyMpL`

    """

    Name = ['EvolutionStrategyML', 'EvolutionStrategy(mu,lambda)', 'ES(m,l)']

    @staticmethod
    def info():
        r"""Get algorithms information.

        Returns:
            str: Algorithm information.

        See Also:
            * :func:`niapy.algorithms.Algorithm.info`

        """
        return r"""TODO"""

    def new_pop(self, pop):
        r"""Return new population.

        Args:
            pop (numpy.ndarray): Current population.

        Returns:
            numpy.ndarray: New population.

        """
        pop_s = np.argsort([i.f for i in pop])
        if self.mu < self.lam:
            return objects_to_array([pop[i] for i in pop_s[:self.mu]])
        new_population = list()
        for i in range(int(ceil(float(self.mu) / self.lam))):
            new_population.extend(pop[:self.lam if (self.mu - i * self.lam) >= self.lam else self.mu - i * self.lam])
        return objects_to_array(new_population)

    def init_population(self, task):
        r"""Initialize starting population.

        Args:
            task (Task): Optimization task.

        Returns:
            Tuple[numpy.ndarray[Individual], numpy.ndarray[float], Dict[str, Any]]:
                1. Initialized population.
                2. Initialized populations fitness/function values.
                3. Additional arguments.

        See Also:
            * :func:`niapy.algorithm.basic.es.EvolutionStrategyMpL.init_population`

        """
        c, fc, _ = EvolutionStrategyMpL.init_population(self, task)
        return c, fc, {}

    def run_iteration(self, task, c, population_fitness, best_x, best_fitness, **params):
        r"""Core function of EvolutionStrategyML algorithm.

        Args:
            task (Task): Optimization task.
            c (numpy.ndarray): Current population.
            population_fitness (numpy.ndarray): Current population fitness/function values.
            best_x (numpy.ndarray): Global best individual.
            best_fitness (float): Global best individuals fitness/function value.
            **params Dict[str, Any]: Additional arguments.

        Returns:
            Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray, float, Dict[str, Any]]:
                1. New population.
                2. New populations fitness/function values.
                3. New global best solution.
                4. New global best solutions fitness/objective value.
                5. Additional arguments.

        """
        cn = objects_to_array(
            [IndividualES(x=self.mutate_rand(c, task), task=task, rand=self.rng) for _ in range(self.lam)])
        c = self.new_pop(cn)
        fc = np.asarray([x.f for x in c])
        best_x, best_fitness = self.get_best(c, fc, best_x, best_fitness)
        return c, fc, best_x, best_fitness, {}



# encoding=utf8

"""Python micro framework for building nature-inspired algorithms."""

from niapy import util, algorithms, problems, task
from niapy.runner import Runner

__all__ = ["algorithms", "problems", "util", "task", "Runner"]
__project__ = "NiaPy"
__version__ = "2.0.1"

VERSION = "{0} v{1}".format(__project__, __version__)



# encoding=utf8

"""Sphere problems."""

import numpy as np
from niapy.problems.problem import Problem

__all__ = ['Sphere', 'Sphere2', 'Sphere3']


class Sphere(Problem):
    r"""Implementation of Sphere functions.

    Date: 2018

    Authors: Iztok Fister Jr.

    License: MIT

    Function: **Sphere function**

        :math:`f(\mathbf{x}) = \sum_{i=1}^D x_i^2`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [0, 10]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (0,...,0)`

    LaTeX formats:
        Inline:
            $f(\mathbf{x}) = \sum_{i=1}^D x_i^2$

        Equation:
            \begin{equation}f(\mathbf{x}) = \sum_{i=1}^D x_i^2 \end{equation}

        Domain:
            $0 \leq x_i \leq 10$

    Reference paper:
        Jamil, M., and Yang, X. S. (2013).
        A literature survey of benchmark functions for global optimisation problems.
        International Journal of Mathematical Modelling and Numerical Optimisation,
        4(2), 150-194.

    """

    def __init__(self, dimension=4, lower=-5.12, upper=5.12, *args, **kwargs):
        r"""Initialize Sphere problem..

        Args:
            dimension (Optional[int]): Dimension of the problem.
            lower (Optional[Union[float, Iterable[float]]]): Lower bounds of the problem.
            upper (Optional[Union[float, Iterable[float]]]): Upper bounds of the problem.

        See Also:
            :func:`niapy.problems.Problem.__init__`

        """
        super().__init__(dimension, lower, upper, *args, **kwargs)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\mathbf{x}) = \sum_{i=1}^D x_i^2$'''

    def _evaluate(self, x):
        return np.sum(x ** 2)


class Sphere2(Problem):
    r"""Implementation of Sphere with different powers function.

    Date: 2018

    Authors: Klemen Berkovič

    License: MIT

    Function: **Sun of different powers function**

        :math:`f(\textbf{x}) = \sum_{i = 1}^D \lvert x_i \rvert^{i + 1}`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-1, 1]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (0,...,0)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = \sum_{i = 1}^D \lvert x_i \rvert^{i + 1}$

        Equation:
            \begin{equation} f(\textbf{x}) = \sum_{i = 1}^D \lvert x_i \rvert^{i + 1} \end{equation}

        Domain:
            $-1 \leq x_i \leq 1$

    Reference URL:
        https://www.sfu.ca/~ssurjano/sumpow.html

    """

    def __init__(self, dimension=4, lower=-1.0, upper=1.0, *args, **kwargs):
        r"""Initialize Sphere2 problem..

        Args:
            dimension (Optional[int]): Dimension of the problem.
            lower (Optional[Union[float, Iterable[float]]]): Lower bounds of the problem.
            upper (Optional[Union[float, Iterable[float]]]): Upper bounds of the problem.

        See Also:
            :func:`niapy.problems.Problem.__init__`

        """
        super().__init__(dimension, lower, upper, *args, **kwargs)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = \sum_{i = 1}^D \lvert x_i \rvert^{i + 1}$'''

    def _evaluate(self, x):
        indices = np.arange(2, self.dimension + 2)
        return np.sum(np.power(np.abs(x), indices))


class Sphere3(Problem):
    r"""Implementation of rotated hyper-ellipsoid function.

    Date: 2018

    Authors: Klemen Berkovič

    License: MIT

    Function: **Sun of rotated hyper-ellipsoid function**

        :math:`f(\textbf{x}) = \sum_{i = 1}^D \sum_{j = 1}^i x_j^2`

        **Input domain:**
        The function can be defined on any input domain but it is usually
        evaluated on the hypercube :math:`x_i ∈ [-65.536, 65.536]`, for all :math:`i = 1, 2,..., D`.

        **Global minimum:** :math:`f(x^*) = 0`, at :math:`x^* = (0,...,0)`

    LaTeX formats:
        Inline:
            $f(\textbf{x}) = \sum_{i = 1}^D \sum_{j = 1}^i x_j^2$

        Equation:
            \begin{equation} f(\textbf{x}) = \sum_{i = 1}^D \sum_{j = 1}^i x_j^2 \end{equation}

        Domain:
            $-65.536 \leq x_i \leq 65.536$

    Reference URL:
        https://www.sfu.ca/~ssurjano/rothyp.html

    """

    def __init__(self, dimension=4, lower=-65.536, upper=65.536, *args, **kwargs):
        r"""Initialize Sphere3 problem..

        Args:
            dimension (Optional[int]): Dimension of the problem.
            lower (Optional[Union[float, Iterable[float]]]): Lower bounds of the problem.
            upper (Optional[Union[float, Iterable[float]]]): Upper bounds of the problem.

        See Also:
            :func:`niapy.problems.Problem.__init__`

        """
        super().__init__(dimension, lower, upper, *args, **kwargs)

    @staticmethod
    def latex_code():
        r"""Return the latex code of the problem.

        Returns:
            str: Latex code.

        """
        return r'''$f(\textbf{x}) = \sum_{i = 1}^D \sum_{j = 1}^i x_j^2$'''

    def _evaluate(self, x):
        x_matrix = np.tile(x, (self.dimension, 1))
        val = np.sum(np.tril(x_matrix) ** 2.0, axis=0)
        return np.sum(val)
